﻿// Ionic Starter App
angular.module('mallista-app', ['ionic', 'ngCordova', 'ionic-datepicker', 'mallista.controllers', 'mallista.i18n.service',
                            'shopping.list.helper.service', 'mallista.database.service',
                            'mallista.setting.service', 'mallista.items', 'mallista.tool.service',
                            'pending.helper.service', 'product.helper.service', 'mallista.image.service',
                            'mallista.statistics.service', 'maintenance.helper.service'])
.run(['$ionicPlatform', '$rootScope', '$q', 'TranslatorService', 'SettingService', 'mallistaDB',
    function ($ionicPlatform, $rootScope, $q, TranslatorService, SettingService, mallistaDB) {
        $ionicPlatform.ready(function () {
            //Indicate that application is starting up
            $rootScope.initializing = true;

            // Hide the accessory bar by default (remove this to show the accessory bar above the keyboard
            // for form inputs)
            if (window.cordova && window.cordova.plugins && window.cordova.plugins.Keyboard) {
              cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
            }
            if (window.StatusBar) {
              // org.apache.cordova.statusbar required
              StatusBar.styleLightContent();
            }
            //Initialize database and get selected language
            var q = $q.defer();
            $q.when(mallistaDB.createDatabase(), function () {
                //Recover the current language
                //and report it to TranslatorService
                SettingService.getSetting(SettingService.languageKey).then(function (res) {
                    TranslatorService.setCurrentLanguage(res);
                    $rootScope.initializing = false;
                    q.resolve();
                }, function (err) {
                    $rootScope.initializing = false;
                    q.reject();
                });
                q.promise;
            }, function (err) {
                q.reject();
            });
            q.promise;
          });
        }])

.config(['$stateProvider', '$urlRouterProvider', '$ionicConfigProvider',
function ($stateProvider, $urlRouterProvider, $ionicConfigProvider) {

    //Definition of all states
    //Tabs
    $stateProvider
        .state("tabs", {
            url: "/tab",
            abstract: true,
            templateUrl: "tabs.html",
        })
    //List of Shopping lists
    .state("tabs.lists", {
        url: "/lists",
        views: {
            "lists-tab": {
                templateUrl: "lists.html",
                controller: "listsCtrl",
            }
        }
    })
    //Creation of shopping list
    .state("tabs.createlist", {
        url: "/createlist",
        views: {
            "lists-tab": {
                templateUrl: "createlist.html",
                controller: "createListCtrl",
            }
        }
    })
    //Visualisation of shopping list
    .state("tabs.showlist", {
        url: "/showlist",
        views: {
            "lists-tab": {
                templateUrl: "showlist.html",
                controller: "createListCtrl",
            }
        }
    })
    //List of pending products
    .state("tabs.pending", {
        url: "/pending",
        views: {
            "pending-tab": {
                templateUrl: "pending.html",
                controller: "pendingCtrl",
            }
        }
    })
    //Creation of pending product
    .state("tabs.createpending", {
        url: "/createpending",
        views: {
            "pending-tab": {
                templateUrl: "createpending.html",
                controller: "createPendingCtrl",
            }
        }
    })
    //Edition of pending product
    .state("tabs.editpending", {
        url: "/editpending",
        views: {
            "pending-tab": {
                templateUrl: "editpending.html",
                controller: "editPendingCtrl",
            }
        }
    })
    //Visualisation of pending product
    .state("tabs.showpending", {
        url: "/showpending",
        views: {
            "pending-tab": {
                templateUrl: "showpending.html",
                controller: "showPendingCtrl",
            }
        }
    })
    //Buying of pending product
    .state("tabs.buypending", {
        url: "/buypending",
        views: {
            "pending-tab": {
                templateUrl: "buypending.html",
                controller: "buyPendingCtrl",
            }
        }
    })
    //List of products
    .state('tabs.products', {
        url: "/products",
        views: {
            'products-tab': {
                templateUrl: "products.html",
                controller: "productsCtrl",
            }
        }
    })
    //Creation of product
    .state("tabs.createproduct", {
        url: "/createproduct",
        views: {
            "products-tab": {
                templateUrl: "createproduct.html",
                controller: "createProductCtrl",
            }
        }
    })
    //Visualisation of product
    .state("tabs.showproduct", {
        url: "/showproduct",
        views: {
            "products-tab": {
                templateUrl: "showproduct.html",
                controller: "showProductCtrl",
            }
        }
    })
    //Statistics' view
    .state("statistics", {
        abstract: true,
        url: "/statistics",
        templateUrl: "statistics.html",
    })
    //Menu statistics
    .state("statistics.menu", {
        url: "/menustatistics",
        views: {
            "statistics-view": {
                templateUrl: "menustatistics.html",
                controller: "statisticsCtrl",
            }
        }
    })
    //Selection of date range
    .state("statistics.range", {
        url: "/reportrange",
        views: {
            "statistics-view": {
                templateUrl: "reportrange.html",
                controller: "statisticsCtrl",
            }
        }
    })
    //Visualisation of Report
    .state("statistics.result", {
        url: "/reportresult",
        views: {
            "statistics-view": {
                templateUrl: "reportresult.html",
                controller: "statisticsCtrl",
            }
        }
    })
    //Settings' view
    .state("settings", {
        url: "/settings",
        templateUrl: "settings.html",
        controller: "settingsCtrl",
    })
    //Maintenance's view
    .state("maintenance", {
        abstract: true,
        url: "/maintenance",
        templateUrl: "maintenance.html",
    })
    //Menu maintenance
    .state("maintenance.menu", {
        url: "/menumaintenance",
        views: {
            "maintenance-view": {
                templateUrl: "menumaintenance.html",
                controller: "maintenanceCtrl",
            }
        }
    })
    //Table
    .state("maintenance.table", {
        url: "/table",
        views: {
            "maintenance-view": {
                templateUrl: "table.html",
                controller: "maintenanceTableCtrl",
            }
        }
    })
    //Show table register
    .state("maintenance.showregister", {
        url: "/showregister",
        views: {
            "maintenance-view": {
                templateUrl: "showregister.html",
                controller: "maintenanceTableCtrl",
            }
        }
    })
    //Create register
    .state("maintenance.createregister", {
        url: "/createregister",
        views: {
            "maintenance-view": {
                templateUrl: "createregister.html",
                controller: "maintenanceTableCtrl",
            }
        }
    })
    //About's view
    .state("about", {
        url: "/about",
        templateUrl: "about.html",
        controller: "aboutCtrl",
    })
    ;//end states

    //Default view
    $urlRouterProvider.otherwise("/tab/pending");
    //Put tab bar position
    $ionicConfigProvider.tabs.position("bottom");

}]);