﻿angular.module('maintenance.helper.service', [])

.factory('MaintenanceHelper', ['$q', '$ionicLoading', 'mallistaDB', 'TranslatorService', 'ToolService',
    function ($q, $ionicLoading, mallistaDB, TranslatorService, ToolService) {
    var self = this;

    //Insert a new register in a specific table
    self._addRegister = function (table, item) {
        var q = $q.defer();
        var query = null;
        var params = null;

        //Convert activated to a number
        item.activated = ToolService.convertBooleanToNumber(item.activated);

        switch (table) {
            case "Brands":
            case "Categories":
                query = "INSERT INTO {0} (activated, name) VALUES (?, ?);";
                params = [item.activated, item.name];
                break;
            case "Establishments":
                query = "INSERT INTO {0} (activated, image, latitude, longitude,  name, remarks) VALUES (?, ?, ?, ?, ?, ?);";
                params = [item.activated, item.image, item.latitude, item.longitude, item.name, item.remarks];
                break;
            case "Products":
                query = "INSERT INTO {0} (activated, brandId, categoryId, code, establishmentId, image, name, remarks) VALUES (?, ?, ?, ?, ?, ?, ?, ?);";
                params = [item.activated, item.brandId, item.categoryId, item.code, item.establishmentId, item.image, item.name, item.remarks];
                break;
            case "ShoppingLists":
                query = "INSERT INTO {0} (activated, name, remarks) VALUES (?, ?, ?);";
                params = [item.activated, item.name, item.remarks];
                break;
            case "Purchases":
                query = "INSERT INTO {0} (shoppingListId, productId, brandId, establishmentId, totalAmount, unitPrice) VALUES (?, ?, ?, ?, ?, ?);";
                params = [item.shoppingListId, item.productId, item.brandId, item.establishmentId, item.totalAmount, item.unitPrice];
                break;
            case "Settings":
                query = "INSERT INTO {0} (clau, valor) VALUES (?, ?);";
                params = [item.clau, item.valor];
                break;
            case "ShoppingListProducts":
                query = "INSERT INTO {0} (shoppingListId, productId, purchasedQty, requestedQty) VALUES (?, ?, ?, ?);";
                params = [item.shoppingListId, item.productId, item.purchasedQty, item.requestedQty];
                break;
        }

        query = TranslatorService.format(query, table);
        ToolService.showLoading("SAVING_DATA");
        mallistaDB.executeSql(query, params).then(function (res) {
            $ionicLoading.hide();
            q.resolve();
        }, function (err) {
            $ionicLoading.hide();
            q.reject(err);
        });
        return q.promise;
    }
    //Delete a specific register
    self._deleteRegister = function (table, id) {
        var q = $q.defer();
        var query = "DELETE FROM {0} WHERE {1} = ?;";
        var fieldId = null;

        switch (table) {
            case "Brands":
                fieldId = "brandId";
                break;
            case "Categories":
                fieldId = "categoryId";
                break;
            case "Establishments":
                fieldId = "establishmentId";
                break;
            case "Products":
                fieldId = "productId";
                break;
            case "Purchases":
                fieldId = "purchaseId";
                break;
            case "Settings":
                fieldId = "rowId";
                break;
            case "ShoppingListProducts":
                fieldId = "rowId";
                break;
            case "ShoppingLists":
                fieldId = "shoppingListId";
                break;
            default:
                fieldld = "id";
                break;
        }

        query = TranslatorService.format(query, [table, fieldId]);


        ToolService.showLoading("DELETING_DATA");
        mallistaDB.executeSql(query, [id]).then(function (res) {
            $ionicLoading.hide();
            q.resolve();
        }, function (err) {
            $ionicLoading.hide();
            q.reject(err);
        });
        return q.promise;
    }
    //Get the field list for a specific table
    self._getTableFieldList = function (table) {
        var result = null;
        switch (table) {
            case "Brands":
            case "Categories":
                result = ["id", "activated", "name"];
                break;
            case "Establishments":
                result = ["id", "activated", "image", "latitude", "longitude", "name", "remarks"];
                break;
            case "Products":
                result = ["id", "activated", "brandId", "categoryId", "code", "creationDate", "establishmentId", "image", "modificationDate", "name", "remarks"];
                break;
            case "ShoppingLists":
                result = ["id", "activated", "creationDate", "modificationDate", "name", "remarks"];
                break;
            case "Purchases":
                result = ["id", "shoppingListId", "productId", "brandId", "establishmentId", "purchaseDate", "purchasedQty", "totalAmount", "unitPrice"];
                break;
            case "Settings":
                result = ["id", "clau", "valor"];
                break;
            case "ShoppingListProducts":
                result = ["id", "shoppingListId", "productId", "creationDate", "modificationDate", "purchasedQty", "requestedQty"];
                break;
        }
        return result;
    }
    //Get all register from a specific table
    self._getRegisterList = function (table) {
        var q = $q.defer();
        var fieldId = null;
        var fieldDescription = "name";
        var orderBy = fieldDescription + " COLLATE NOCASE ASC";

        var query = "SELECT t.{0} AS id, t.{1} AS description, t.*" +
                    " FROM {2} t" +
                    " ORDER BY t.{3};";

        var queryExtended = "SELECT t.{0} AS id" +
                                ", t.{1} AS description" +
                                ", strftime('%d/%m/%Y %H:%M', t.creationDate) AS cDate" +
                                ", strftime('%d/%m/%Y %H:%M', t.modificationDate) AS mDate" +
                                ", t.*" +
                            " FROM {2} t" +
                            " ORDER BY t.{3};";

        var queryPending = "SELECT t.{0} AS id" +
                                ", {1} AS description" +
                                ", '1' AS activated" +
                                ", strftime('%d/%m/%Y %H:%M', t.creationDate) AS cDate" +
                                ", strftime('%d/%m/%Y %H:%M', t.modificationDate) AS mDate" +
                                ", sp.name AS shoppingListName" +
                                ", t.*" +
                            " FROM {2} t" +
                            " LEFT JOIN Products p" +
                            " ON t.productId = p.productId" +
                            " LEFT JOIN ShoppingLists sp" +
                            " ON t.shoppingListId = sp.shoppingListId" +
                            " ORDER BY t.{3} DESC;";

        var queryPurchase = "SELECT t.{0} AS id" +
                                ", {1} AS description" +
                                ", '1' AS activated" +
                                ", strftime('%d/%m/%Y %H:%M', t.purchaseDate) AS pDate" +
                                ", t.*" +
                            " FROM {2} t" +
                            " LEFT JOIN Products p" +
                            " ON t.productId = p.productId" +
                            " ORDER BY t.{3} DESC;";

        switch (table) {
            case "Brands":
                fieldId = "brandId";
                break;
            case "Categories":
                fieldId = "categoryId";
                break;
            case "Establishments":
                fieldId = "establishmentId";
                break;
            case "Products":
                fieldId = "productId";
                query = queryExtended;
                break;
            case "Purchases":
                fieldId = "purchaseId";
                fieldDescription = "p.name"
                orderBy = fieldId;
                query = queryPurchase;
                break;
            case "Settings":
                fieldId = "rowId";
                break;
            case "ShoppingListProducts":
                fieldId = "rowId";
                fieldDescription = "p.name"
                orderBy = fieldId;
                query = queryPending;
                break;
            case "ShoppingLists":
                fieldId = "shoppingListId";
                query = queryExtended;
                break;
            default:
                fieldld = "id";
                break;
        }

        var result = [];
        query = TranslatorService.format(query, [fieldId, fieldDescription, table, orderBy]);

        ToolService.showLoading("LOADING_DATA");
        mallistaDB.executeSql(query).then(function (res) {
            if (res.rows.length > 0) {
                for (var i = 0; i < res.rows.length; i++) {
                    var item = res.rows.item(i);
                    result.push(item);
                }
            }
            $ionicLoading.hide();
            q.resolve(result);
        }, function (err) {
            $ionicLoading.hide();
            q.reject(err);
        });
        return q.promise;
    }
    //Indicate whether a table can delete registers
    self._tableCanDelete = function (table) {
        var result = false;
        switch (table) {
            case "Purchases":
            case "Settings":
                result = true;
                break;
            default:
                result = false;
                break;
        }
        return result;
    }
        //Update a register from a specific table
    self._updateRegister = function (table, item) {
        var q = $q.defer();
        var query = null;
        var params = null;

        //Convert activated to a number
        item.activated = ToolService.convertBooleanToNumber(item.activated);

        switch (table) {
            case "Brands":
                query = "UPDATE Brands" +
                            " SET activated = ?, name = ?" +
                        " WHERE brandId = ?;";
                params = [item.activated, item.name, item.id];
                break;

            case "Categories":
                query = "UPDATE Categories" +
                            " SET activated = ?, name = ?" +
                        " WHERE categoryId = ?;";
                params = [item.activated, item.name, item.id];
                break;

            case "Establishments":
                query = "UPDATE Establishments" +
                            " SET activated = ?, image = ?, latitude = ?, longitude = ?" +
                                ", name = ?, remarks = ?" +
                        " WHERE establishmentId = ?;";
                params = [item.activated, item.image, item.latitude, item.longitude
                            , item.name, item.remarks, item.id];
                break;

            case "Products":
                query = "UPDATE Products" +
                            " SET activated = ?, brandId = ?, categoryId = ?, code = ?" +
                                ", establishmentId = ?, image = ?" +
                                ", name = ?, remarks = ?" +
                                ", modificationDate = datetime('now', 'localtime')" +
                        " WHERE productId = ?;";
                params = [item.activated, item.brandId, item.categoryId, item.code
                            , item.establishmentId, item.image, item.name, item.remarks, item.id];
                break;

            case "ShoppingLists":
                query = "UPDATE ShoppingLists" +
                            " SET activated = ?, name = ?, remarks = ?" +
                                ", modificationDate = datetime('now', 'localtime')" +
                        " WHERE shoppingListId = ?;";
                params = [item.activated, item.name, item.remarks, item.id];
                break;

            case "Purchases":
                query = "UPDATE Purchases" +
                            " SET shoppingListId = ?, productId = ?, brandId = ?" +
                                ", establishmentId = ?, purchaseDate = datetime(?), purchasedQty = ?" +
                                ", totalAmount = ?, unitPrice = ?" +
                        " WHERE purchaseId = ?;";
                params = [item.shoppingListId, item.productId, item.brandId
                            , item.establishmentId, item.purchaseDate, item.purchasedQty
                            , item.totalAmount, item.unitPrice, item.id];
                break;

            case "Settings":
                query = "UPDATE Settings" +
                            " SET clau = ?, valor = ?" +
                        " WHERE rowId = ?;";
                params = [item.clau, item.valor, item.id];
                break;

            case "ShoppingListProducts":
                query = "UPDATE ShoppingListProducts" +
                            " SET shoppingListId = ?, productId = ?" +
                                ", purchasedQty = ?, requestedQty = ?" +
                                ", modificationDate = datetime('now', 'localtime')" +
                        " WHERE rowId = ?;";
                params = [item.shoppingListId, item.productId
                            , item.purchasedQty, item.requestedQty, item.id];
                break;

        }

        ToolService.showLoading("SAVING_DATA");
        mallistaDB.executeSql(query, params, true).then(function (res) {
            $ionicLoading.hide();
            q.resolve();
        }, function (err) {
            $ionicLoading.hide();
            ToolService.showMessage(err.message);
            q.reject(err);
        });
        return q.promise;
    }

        //Service's public methods
    return {
        addRegister: function (table, item) {
            return self._addRegister(table, item);
        },
        deleteRegister: function (table, id){
            return self._deleteRegister(table, id);
        },
        getTableFieldList: function (table) {
            return self._getTableFieldList(table);
        },
        getRegisterList: function (table) {
            return self._getRegisterList(table);
        },
        tableCanDelete: function (table) {
            return self._tableCanDelete(table);
        },
        updateRegister: function (table, item) {
            return self._updateRegister(table, item);
        },
    };
}])

.filter('maintenanceFilter', [function () {
    return function (items, expression) {
        var foundItems = [];

        if (expression != undefined) {
            expression = expression.toLowerCase();

            angular.forEach(items, function (item) {
                if (item.description != undefined) {
                    if (item.description.toLowerCase().indexOf(expression) >= 0)
                        foundItems.push(item);
                }
            });
        }
        return foundItems;
    };
}])
; //end module


