﻿angular.module('mallista.controllers', [])

//#region SIDE MENU CONTROLLER
    .controller('sideMenuCtrl',['$scope', '$rootScope', '$ionicSideMenuDelegate', 'TranslatorService', function ($scope, $rootScope, $ionicSideMenuDelegate, TranslatorService) {
        $scope = $rootScope;
        $scope.showMenu = function () {
            $ionicSideMenuDelegate.toggleLeft();
        };
        $scope.t = function (token) {
            return TranslatorService.getToken(token);
        }
    }])
//#endregion SIDE MENU CONTROLLER

//#region PENDING CONTROLLER
    .controller('pendingCtrl', ['$scope', '$location', '$ionicActionSheet', '$ionicPopup', 'TranslatorService', 'ShoppingListHelper', 'PendingHelper', 'ToolService', 'PendingItem', 'SettingService', 
    function ($scope, $location, $ionicActionSheet, $ionicPopup, TranslatorService, ShoppingListHelper, PendingHelper, ToolService, PendingItem, SettingService) {
        $scope.pendingList = [];
        $scope.shoppingLists = [];
        $scope.selectedShoppingList = null;
        $scope.selectedItem = {};
        $scope.settings = [];

        //Edit the selected list
        $scope.addPending = function (shoppingListId) {
            var data = PendingItem;
            data.clear();
            data.shoppingListId = shoppingListId;
            data.valid = true;
            $location.path("/tab/createpending");
        }
        //Buy a pending product
        $scope.buyPending = function (item){
            var data = PendingItem;
            data.clear();
            data.id = item.id;
            data.productId = item.productId;
            data.shoppingListId = item.shoppingListId;
            data.brandId = item.brandId;
            data.establishmentId = item.establishmentId;
            data.isEdition = true;
            data.productName = item.productName;
            data.requestedQty = item.requestedQty;
            data.shoppingListName = item.shoppingListName;
            data.valid = true;
            $location.path("/tab/buypending");
        }
        //Show a popup in order to confirm deletion
        $scope.confirmDeletePending = function (item) {
            var confirmPopup = $ionicPopup.confirm({
                title: TranslatorService.getToken('DELETE_PENDING_TITLE'),
                template: TranslatorService.getToken('DELETE_PENDING_BODY', [item.productName, item.shoppingListName])
            });
            confirmPopup.then(function (res) {
                if (res) {
                    $scope.deletePending(item);
                }
            });
        }
        //Delete the selected pending product
        $scope.deletePending = function (item) {
            PendingHelper.deletePending(item.id).then(function () {
                $scope.refresh();
            });
        }
        //Edit the selected pending product
        $scope.editPending = function (item) {
            var data = PendingItem;
            data.clear();
            data.id = item.id;
            data.productId = item.productId;
            data.shoppingListId = item.shoppingListId;
            data.isEdition = true;
            data.productName = item.productName;
            data.requestedQty = item.requestedQty;
            data.valid = true;
            $location.path("/tab/editpending");
        }
        //Get the image's url
        $scope.getImage = function (url) {
            if ($scope.settings.showPicture)
                return ToolService.getImage(url);
            else
                return "";
        }
        //Get settings for this view
        $scope.getSettings = function () {
            //Recover the current language
            //and report it to TranslatorService
            SettingService.getSetting(SettingService.languageKey).then(function (res) {
                TranslatorService.setCurrentLanguage(res);
            });
            SettingService.getSetting(SettingService.pendingShowPicture, false).then(function (res) {
                $scope.settings.showPicture = ToolService.isTrue(res);
            });
        }
        //Get the list of shopping lists 
        $scope.getShoppingLists = function () {
            ShoppingListHelper.getShoppingLists(true).then(function (res) {
                $scope.shoppingLists = res;
                if ($scope.selectedShoppingList == null)
                    $scope.selectedShoppingList = $scope.shoppingLists[0];
            }, function (err) {
                $scope.shoppingLists = [];
            });
        }
        //Refresh data (fired from mallistaRefresh directive)
        $scope.refresh = function () {
            $scope.getSettings();
            PendingHelper.getPendingList().then(function (res) {
                $scope.pendingList = res;
            }, function (err) {
                $scope.pendingList = [];
            });
            $scope.getShoppingLists();
        }
        //Set the current selection
        $scope.selectItem = function (item) {
            $scope.selectedItem = item;
        }
        //Show de context menu for selected shopping list
        $scope.showContextMenu = function (item) {
            $scope.selectedItem = item;
            //Show the action sheet
            var hideSheet = $ionicActionSheet.show({
                buttons: [
                    { text: TranslatorService.getToken("BUY") },
                    { text: TranslatorService.getToken("VIEW_DETAILS") },
                    { text: TranslatorService.getToken("EDIT") },
                    { text: TranslatorService.getToken("DELETE") }
                ],
                titleText: '',
                cssClass: 'mallista-action-sheet',
                buttonClicked: function (index) {
                    if (index == 0) {
                        $scope.buyPending(item);
                    } else if (index == 1){
                        $scope.showPending(item);
                    } else if (index == 2) {
                        $scope.editPending(item);
                    } else if (index == 3) {
                        $scope.confirmDeletePending(item);
                    }
                    return true;
                }
            });
        }
        //Edit the selected product
        $scope.showPending = function (item) {
            var data = PendingItem;
            data.clear();
            data.id = item.id;
            data.brandName = item.brandName;
            data.categoryName = item.categoryName;
            data.code = item.code;
            data.creationDate = item.creationDate;
            data.establishmentName = item.establishmentName;
            data.image = item.image;
            data.isEdition = false;
            data.modificationDate = item.modificationDate;
            data.productName = item.productName;
            data.productRemarks = item.productRemarks;
            data.purchasedQty = item.purchasedQty;
            data.requestedQty = item.requestedQty;
            data.shoppingListName = item.shoppingListName;
            data.shoppingListRemarks = item.shoppingListRemarks;
            data.valid = true;
            $location.path("/tab/showpending");
        }
        //Get the corresponding translation for a token
        $scope.t = function (token) {
            return TranslatorService.getToken(token);
        }
    }])
//#endregion PENDING CONTROLLER

//#region CREATE PENDING CONTROLLER
    .controller('createPendingCtrl', ['$scope', '$q', '$ionicActionSheet', '$ionicPopup', '$location', '$ionicModal',
                                    'ProductItem', 'TranslatorService', 'ShoppingListHelper', 'ProductHelper',
                                    'ToolService', 'PendingHelper', 'PendingItem', 'SettingService',
    function ($scope, $q, $ionicActionSheet, $ionicPopup, $location, $ionicModal, ProductItem, TranslatorService,
                    ShoppingListHelper, ProductHelper, ToolService, PendingHelper, PendingItem, SettingService) {
        $scope.searcherInput = "";
        $scope.shoppingLists = [];
        $scope.selectedItem = {};
        $scope.pending = {};
        $scope.shoppingListId = 0;
        $scope.settings = [];

        $ionicModal.fromTemplateUrl('addpendingform.html', {
            scope: $scope
        }).then(function (modal) {
            $scope.modal = modal;
        });
        //Get settings for this view
        getSettings = function () {
            SettingService.getSetting(SettingService.productShowPicture, false).then(function (res) {
                $scope.settings.showPicture = ToolService.isTrue(res);
            });
        }
        //Get the list of shopping list 
        getShoppingLists = function () {
            ShoppingListHelper.getShoppingLists().then(function (res) {
                $scope.shoppingLists = res;
                angular.forEach($scope.shoppingLists, function (item) {
                    if (item.id == $scope.shoppingListId)
                        $scope.pending.shoppingList = item;
                });
            }, function (err) {
                $scope.shoppingLists = [];
            });
        }
        //Add a new product
        $scope.addPending = function (formValid, productId, shoppingListId, quantity) {
            //Validate form
            if (formValid == true) {
                $q.when(PendingHelper.addPending(productId, shoppingListId, quantity), function () {
                    $scope.modal.hide();
                    $location.path("/tab/pending");
                }, function (err) {
                    ToolService.showMessage("UNEXPECTED_ERROR");
                });
            } else {
                ToolService.showMessage("INVALID_FORM");
            }
        }
        //Clean the text on searcher input control
        $scope.cleanSearcherInput = function () {
            $scope.searcherInput = "";
        }
        //Get the image's url
        $scope.getImage = function (url) {
            if ($scope.settings.showPicture)
                return ToolService.getImage(url);
            else
                return "";
        }
        //Refresh data (fired from mallistaRefresh directive)
        $scope.refresh = function () {
            getSettings();
            ProductHelper.getProductList().then(function (res) {
                $scope.products = res;
            }, function (err) {
                $scope.products = [];
            });
            getShoppingLists();
        }
        //Set the current selection
        $scope.selectItem = function (item) {
            $scope.selectedItem = item;
        }
        //Setup view (fired from mallistaSetup directive)
        $scope.setup = function () {
            //Recover data from previous view
            var item = PendingItem;
            if (item.valid == true) {
                $scope.shoppingListId = item.shoppingListId;
                item.valid = false;
            }
        }
        //Show de context form for selected pending product
        $scope.showContextForm = function (item) {
            $scope.selectedItem = item;
            $scope.pending.quantity = 1;
            $scope.modal.show();
        }
        //Get the corresponding translation for a token
        $scope.t = function (token) {
            return TranslatorService.getToken(token);
        }
    }])
//#endregion CREATE PENDING CONTROLLER

//#region EDIT PENDING CONTROLLER
    .controller('editPendingCtrl', ['$scope', '$q', '$location', 'ToolService',
                'TranslatorService', 'ShoppingListHelper', 'PendingHelper',
                'PendingItem',
    function ($scope, $q, $location, ToolService, TranslatorService,
                ShoppingListHelper, PendingHelper, PendingItem) {
        $scope.pending = [];
        $scope.shoppingLists = [];

        //Get the list of shopping list 
        $scope.getShoppingLists = function () {
            ShoppingListHelper.getShoppingLists().then(function (res) {
                $scope.shoppingLists = res;
                angular.forEach(res, function (item) {
                    if (item.id == $scope.pending.shoppingListId) {
                        $scope.pending.shoppingList = item;
                    }
                });
            }, function (err) {
                $scope.shoppingLists = [];
            });
        }
        //Indicate whether a product exists on database
        isRegistered = function (name) {
            q = $q.defer();

            $q.when(ProductHelper.isRegistered(name), function (res) {
                q.resolve(res == true);
            }, function () {
                q.reject(false);
            })
            return q.promise;
        }
        //Save a product
        $scope.savePending = function (formValid, shoppingListId, productId, quantity) {
            if ($scope.pending.id != undefined && $scope.pending.id > 0) {
                $scope.updatePending(formValid, $scope.pending.id, shoppingListId, productId, quantity);
            }
        }
        //Setup view (fired from mallistaSetup directive)
        $scope.setup = function () {
            //Recover data from previous view
            var item = PendingItem;
            if (item.valid == true) {
                $scope.pending.id = item.id;
                $scope.pending.productId = item.productId;
                $scope.pending.productName = item.productName;
                $scope.pending.requestedQty = item.requestedQty;
                $scope.pending.shoppingListId = item.shoppingListId;
                item.valid = false;
            }
        }
        //Get the corresponding translation for a token
        $scope.t = function (token) {
            return TranslatorService.getToken(token);
        }
        //Update an specific product
        $scope.updatePending = function (formValid, id, shoppingListId, productId, quantity) {
            //Validate form
            if (formValid == true) {
                $q.when(PendingHelper.updatePending(id, shoppingListId, productId, quantity), function () {
                    $location.path("/tab/pending");
                }, function (err) {
                    ToolService.showMessage("UNEXPECTED_ERROR");
                });
            } else {
                ToolService.showMessage("INVALID_FORM");
            }
        }
        //Load data from master tables
        $scope.getShoppingLists();
    }])
//#endregion EDIT PENDING CONTROLLER

//#region SHOW PENDING CONTROLLER
    .controller('showPendingCtrl', ['$scope', 'TranslatorService', 'PendingItem', 'ToolService',
    function ($scope, TranslatorService, PendingItem, ToolService) {
        $scope.pending = [];

        //Indicate whether a value is empty
        $scope.isEmpty = function (value) {
            return ToolService.isEmpty(value);
        }
        //Setup view (fired from mallistaSetup directive)
        $scope.setup = function () {
            //Recover data from previous view
            var item = PendingItem;
            if (item.valid == true) {
                $scope.pending.id = item.id;
                $scope.pending.brandName = item.brandName;
                $scope.pending.categoryName = item.categoryName;
                $scope.pending.creationDate = item.creationDate;
                $scope.pending.code = item.code;
                $scope.pending.establishmentName = item.establishmentName;
                $scope.pending.image = item.image;
                $scope.pending.modificationDate = item.modificationDate;
                $scope.pending.productName = item.productName;
                $scope.pending.purchasedQty = item.purchasedQty;
                $scope.pending.requestedQty = item.requestedQty;
                $scope.pending.shoppingListName = item.shoppingListName;
                $scope.pending.productRemarks = item.productRemarks;
                $scope.pending.shoppingListRemarks = item.shoppingListRemarks;
                item.valid = false;
            }
        }
        //Get the corresponding translation for a token
        $scope.t = function (token) {
            return TranslatorService.getToken(token);
        }
    }])
//#endregion SHOW PENDING CONTROLLER

//#region BUY PENDING CONTROLLER
    .controller('buyPendingCtrl', ['$scope', '$q', '$location', 'TranslatorService',
        'ProductHelper', 'PendingHelper', 'PendingItem', 'ToolService',
    function ($scope, $q, $location, TranslatorService,
        ProductHelper, PendingHelper, PendingItem, ToolService) {
        $scope.pending = [];
        $scope.brandList = [];
        $scope.establishmentList = [];

        //Buy an specific pending product
        $scope.buyPending = function (formValid, id, shoppingListId, productId, brandId, establishmentId, purchasedQty, unitPrice, totalAmount) {
            //Validate form
            if (formValid == true) {
                $q.when(PendingHelper.buyPending(id, shoppingListId, productId, brandId, establishmentId, purchasedQty, unitPrice, totalAmount), function () {
                    $location.path("/tab/pending");
                }, function (err) {
                    ToolService.showMessage("UNEXPECTED_ERROR");
                });
            } else {
                ToolService.showMessage("INVALID_FORM");
            }
        }
        //Calculate total amount
        $scope.calculateTotalAmount = function () {
            var quantity = $scope.pending.purchasedQty;
            var unitPrice = $scope.pending.unitPrice;
            if (!isNaN(quantity) && !isNaN(unitPrice)) {
                var totalAmount = parseFloat((quantity * Math.round(unitPrice * 1000000)) / 1000000);
                $scope.pending.totalAmount = ToolService.round(totalAmount, 2);
            }
        }
        //Calculate unit price
        $scope.calculateUnitPrice = function () {
            var quantity = $scope.pending.purchasedQty;
            var totalAmount = $scope.pending.totalAmount;
            if (!isNaN(quantity) && !isNaN(totalAmount)) {
                var unitPrice = parseFloat((Math.round(totalAmount * 1000000) / quantity) / 1000000);
                $scope.pending.unitPrice = ToolService.round(unitPrice, 5);
            }
        }
        //Get the brand list 
        $scope.getBrandList = function () {
            ProductHelper.getBrandList(true).then(function (res) {
                $scope.brandList = res;
                angular.forEach(res, function (item) {
                    if (item.id == $scope.pending.brandId) {
                        $scope.pending.brand = item;
                    }
                });
            }, function (err) {
                $scope.brandList = [];
            });
        }
        //Get the establishment list 
        $scope.getEstablishmentList = function () {
            ProductHelper.getEstablishmentList(true).then(function (res) {
                $scope.establishmentList = res;
                angular.forEach(res, function (item) {
                    if (item.id == $scope.pending.establishmentId) {
                        $scope.pending.establishment = item;
                    }
                });
            }, function (err) {
                $scope.establishmentList = [];
            });
        }
        //Save a product
        $scope.savePending = function (formValid, brandId, establishmentId, purchasedQty, unitPrice, totalAmount) {
            if ($scope.pending.id != undefined && $scope.pending.id > 0) {
                $scope.buyPending(formValid, $scope.pending.id, $scope.pending.shoppingListId, $scope.pending.productId, brandId, establishmentId, purchasedQty, unitPrice, totalAmount);
            }
        }
        //Setup view (fired from mallistaSetup directive)
        $scope.setup = function () {
            //Recover data from previous view
            var item = PendingItem;
            if (item.valid == true) {
                $scope.pending.id = item.id;
                $scope.pending.brandId = item.brandId;
                $scope.pending.productId = item.productId;
                $scope.pending.productName = item.productName;
                $scope.pending.establishmentId = item.establishmentId;
                $scope.pending.purchasedQty = item.requestedQty;
                $scope.pending.requestedQty = item.requestedQty;
                $scope.pending.shoppingListId = item.shoppingListId;
                item.valid = false;
            }
            //Load data from master tables
            $scope.getBrandList();
            $scope.getEstablishmentList();
        }
        //Get the corresponding translation for a token
        $scope.t = function (token) {
            return TranslatorService.getToken(token);
        }
    }])
//#endregion BUY PENDING CONTROLLER

//#region SHOPPING LIST CONTROLLER
    .controller('listsCtrl', ['$scope', '$ionicActionSheet', '$ionicPopup', '$location', 'ShoppingListItem',
                                'TranslatorService', 'ShoppingListHelper', 'ToolService',
    function ($scope, $ionicActionSheet, $ionicPopup, $location, ShoppingListItem,
                                TranslatorService, ShoppingListHelper, ToolService) {
        $scope.searcherInput = "";
        $scope.shoppingLists = [];
        $scope.selectedItem = {};

        //Add a new shopping list
        $scope.addShoppingList = function (){
            $location.path("/tab/createlist");
        }
        //Clean the searcher input control
        $scope.cleanSearcherInput = function () {
            $scope.searcherInput = "";
        }
        //Show a popup in order to confirm deletion
        $scope.confirmDeleteList = function (item) {
            var confirmPopup = $ionicPopup.confirm({
                title: TranslatorService.getToken('DELETE_LIST_TITLE'),
                template: TranslatorService.getToken('DELETE_LIST_BODY', item.name)
            });
            confirmPopup.then(function (res) {
                if (res) {
                    $scope.deleteList(item);
                }
            });
        }
        //Delete the selected list
        $scope.deleteList = function (item) {
            ShoppingListHelper.deleteShoppingList(item.id).then(function () {
                $scope.refresh();
            });
        }
        //Edit the selected list
        $scope.editList = function (item, readOnly) {
            var data = ShoppingListItem;
            data.clear();
            data.id = item.id;
            data.creationDate = item.creationDate;
            data.isEdition = true;
            data.name = item.name;
            data.remarks = item.remarks;
            data.valid = true;
            if (readOnly != true) {
                $location.path("/tab/createlist");
            } else {
                $location.path("/tab/showlist");
            }
        }
        //Refresh data (fired from mallistaRefresh directive)
        $scope.refresh = function () {
            ShoppingListHelper.getShoppingLists().then(function (res) {
                $scope.shoppingLists = res;
            }, function (err) {
                $scope.shoppingLists = [];
            });
        }
        //Store the selected shopping list
        $scope.selectItem = function (item){
            $scope.selectedItem = item;
        }
        //Show de context menu for selected shopping list
        $scope.showContextMenu = function (item) {
            $scope.selectedItem = item;
            // Show the action sheet
            var hideSheet = $ionicActionSheet.show({
                buttons: [
                    { text: TranslatorService.getToken("VIEW_DETAILS") },
                    { text: TranslatorService.getToken("EDIT") },
                    { text: TranslatorService.getToken("DELETE") }
                ],
                titleText: '',
                cssClass: 'mallista-action-sheet',
                buttonClicked: function (index) {
                    if (index == 0) {
                        $scope.editList(item, true);
                    } else if (index == 1) {
                        $scope.editList(item);
                    } else if (index == 2) {
                        $scope.confirmDeleteList(item);
                    }
                    return true;
                }
            });
        }
        //Get the corresponding translation for a token
        $scope.t = function (token) {
            return TranslatorService.getToken(token);
        }
    }])
//#endregion SHOPPING LIST CONTROLLER

//#region CREATE SHOPPING LIST CONTROLLER
    .controller('createListCtrl', ['$scope', '$q', '$location', 'TranslatorService', 'ShoppingListHelper', 'ShoppingListItem', 'ToolService',
    function ($scope, $q, $location, TranslatorService, ShoppingListHelper, ShoppingListItem, ToolService) {
        $scope.list = [];
        $scope.list.name = null;
        $scope.list.remarks = null;

        //Indicate whether a shopping list exists on database
        isRegistered = function (name) {
            q = $q.defer();

            $q.when(ShoppingListHelper.isRegistered(name), function (res) {
                q.resolve(res == true);
            }, function () {
                q.reject(false);
            })
            return q.promise;
        }
        //Add a new shopping list
        $scope.addShoppingList = function (formValid, name, remarks) {
            //Validate form
            if (formValid == true) {
                $q.when(isRegistered(name), function (res) {
                    if (res != true) {
                        $q.when(ShoppingListHelper.addShoppingList(name, remarks), function () {
                            $location.path("/tab/lists");
                        }, function (err) {
                            ToolService.showMessage("UNEXPECTED_ERROR");
                        });
                    } else {
                        ToolService.showMessage("ERROR_REGISTERED", name);
                    }
                });
            } else {
                ToolService.showMessage("INVALID_FORM");
            }
        }
        //Indicate whether a value is empty
        $scope.isEmpty = function (value) {
            return ToolService.isEmpty(value);
        }
        //Save a shopping list
        $scope.saveList = function (formValid, name, remarks) {
            if ($scope.list.id != undefined && $scope.list.id > 0) {
                $scope.updateShoppingList(formValid, $scope.list.id, name, remarks);
            } else {
                $scope.addShoppingList(formValid, name, remarks);
            }
        }
        //Setup view (fired from mallistaSetup directive)
        $scope.setup = function () {
            //Recover data from previous view
            var item = ShoppingListItem;
            if (item.valid == true) {
                $scope.list.id = item.id;
                $scope.list.creationDate = item.creationDate;
                $scope.list.isEdition = item.isEdition;
                $scope.list.name = item.name;
                $scope.list.remarks = item.remarks;
                $scope.list.id = item.id;
                item.valid = false;
            }
        }
        ////Get either a value or a space
        //$scope.showValue = function (value){
        //    if (value != undefined && value != null && value != ""){
        //        return value;
        //    } else {
        //        return " ";
        //    }
        //}
        //Get the corresponding translation for a token
        $scope.t = function (token) {
            return TranslatorService.getToken(token);
        }
        //Update a shopping list
        $scope.updateShoppingList = function (formValid, id, name, remarks) {
            //Validate form
            if (formValid == true) {
                $q.when(ShoppingListHelper.updateShoppingList(id, name, remarks), function () {
                    $location.path("/tab/lists");
                }, function (err) {
                    ToolService.showMessage("UNEXPECTED_ERROR");
                });
            } else {
                ToolService.showMessage("INVALID_FORM");
            }
        }
    }])
//#endregion CREATE SHOPPING LIST CONTROLLER

//#region PRODUCT CONTROLLER
    .controller('productsCtrl', ['$scope', '$ionicActionSheet', '$ionicPopup', '$location', 'ProductItem', 'TranslatorService', 'ProductHelper', 'ToolService', 'SettingService',
    function ($scope, $ionicActionSheet, $ionicPopup, $location, ProductItem, TranslatorService, ProductHelper, ToolService, SettingService) {
        $scope.searcherInput = "";
        $scope.products = [];
        $scope.selectedItem = {};
        $scope.settings = [];

        //Add a new product
        $scope.addProduct = function (item) {
            var data = ProductItem;
            data.clear();
            data.valid = true;
            $location.path("/tab/createproduct");
        }
        //Clean the text on searcher input control
        $scope.cleanSearcherInput = function () {
            $scope.searcherInput = "";
        }
        //Ask for confirmation before delete product
        $scope.confirmDeleteProduct = function (item) {
            var confirmPopup = $ionicPopup.confirm({
                title: TranslatorService.getToken('DELETE_PRODUCT_TITLE'),
                template: TranslatorService.getToken('DELETE_PRODUCT_BODY', item.name)
            });
            confirmPopup.then(function (res) {
                if (res) {
                    $scope.deleteProduct(item);
                }
            });
        }
        //Delete the selected product
        $scope.deleteProduct = function (item) {
            ProductHelper.deleteProduct(item.id).then(function () {
                $scope.refresh();
            });
        }
        //Edit the selected product
        $scope.editProduct = function (item, readOnly) {
            var data = ProductItem;
            data.clear();
            data.id = item.id;
            data.brandId = item.brandId;
            data.categoryId = item.categoryId;
            data.code = item.code;
            data.creationDate = item.creationDate;
            data.establishmentId = item.establishmentId;
            data.image = item.image;
            data.isEdition = true;
            data.modificationDate = item.modificationDate;
            data.name = item.name;
            data.remarks = item.remarks;
            data.valid = true;
            if (readOnly != true) {
                $location.path("/tab/createproduct");
            } else {
                $location.path("/tab/showproduct");
            }
        }
        //Get the image's url
        $scope.getImage = function (url) {
            if ($scope.settings.showPicture)
                return ToolService.getImage(url);
            else
                return "";
        }
        //Get settings for this view
        $scope.getSettings = function () {
            SettingService.getSetting(SettingService.productShowPicture, false).then(function (res) {
                $scope.settings.showPicture = ToolService.isTrue(res);
            });
        }
        //Refresh data (fired from mallistaRefresh directive)
        $scope.refresh = function () {
            $scope.getSettings();
            ProductHelper.getProductList().then(function (res) {
                $scope.products = res;
            }, function (err) {
                $scope.products = [];
            });
        }
        //Set the current selection
        $scope.selectItem = function (item) {
            $scope.selectedItem = item;
        }
        //Show the context menu with the action list
        $scope.showContextMenu = function (item) {
            $scope.selectedItem = item;
            // Show the action sheet
            var hideSheet = $ionicActionSheet.show({
                buttons: [
                    { text: TranslatorService.getToken("VIEW_DETAILS") },
                    { text: TranslatorService.getToken("EDIT") },
                    { text: TranslatorService.getToken("DELETE") }
                ],
                titleText: '',
                cssClass: 'mallista-action-sheet',
                buttonClicked: function (index) {
                    if (index == 0) {
                        $scope.editProduct(item, true);
                    } else if (index == 1) {
                        $scope.editProduct(item);
                    } else if (index == 2) {
                        $scope.confirmDeleteProduct(item);
                    }
                    return true;
                }
            });
        }
        //Get the corresponding translation for a token
        $scope.t = function (token) {
            return TranslatorService.getToken(token);
        }
    }])
//#endregion PRODUCT CONTROLLER

//#region CREATE PRODUCT CONTROLLER
    .controller('createProductCtrl', ['$scope', '$q', '$location', 'TranslatorService', 'ProductHelper', 'ProductItem', 'ToolService', 'ImageService',
    function ($scope, $q, $location, TranslatorService, ProductHelper, ProductItem, ToolService, ImageService) {

        $scope.product = [];
        $scope.brands = [];
        $scope.categories = [];
        $scope.establishments = [];

        //Get the brand list
        getBrands = function () {
            $q.when(ProductHelper.getBrandList(), function (res) {
                $scope.brands = res;
            })
        }
        //Get the category list
        getCategories = function () {
            $q.when(ProductHelper.getCategoryList(), function (res) {
                $scope.categories = res;
            })
        }
        //Get the establishment list
        getEstablishments = function () {
            $q.when(ProductHelper.getEstablishmentList(), function (res) {
                $scope.establishments = res;
            })
        }
        //Indicate whether a product exists on database
        isRegistered = function (name) {
            q = $q.defer();

            $q.when(ProductHelper.isRegistered(name), function (res) {
                q.resolve(res == true);
            }, function () {
                q.reject(false);
            })
            return q.promise;
        }
        //Add an image
        $scope.addImage = function (type){
            $q.when(ImageService.addImage(type), function (name) {
                if (name != null && name != undefined && name != "")
                    $scope.product.image = name;
            });
        }
        //Add a new product
        $scope.addProduct = function (formValid, brandId, categoryId, code, establishmentId, image, name, remarks) {
            //Validate form
            if (formValid == true) {
                $q.when(isRegistered(name), function (res) {
                    if (res != true) {
                        $q.when(ProductHelper.addProduct(brandId, categoryId, code, establishmentId, image, name, remarks), function () {
                            $location.path("/tab/products");
                        }, function (err) {
                            ToolService.showMessage("UNEXPECTED_ERROR");
                        });
                    } else {
                        ToolService.showMessage("ERROR_REGISTERED", name);
                    }
                });
            } else {
                ToolService.showMessage("INVALID_FORM");
            }
        }
        //Get either a real image or a dummy image
        $scope.getImage = function (url) {
            return ToolService.getImage(url);
        }
        //Remove the image associated with product
        $scope.removeImage = function (type) {
            $scope.product.image = "";
        }
        //Save a product
        $scope.saveProduct = function (formValid, brandId, categoryId, code, establishmentId, image, name, remarks) {
            if ($scope.product.id != undefined && $scope.product.id > 0) {
                $scope.updateProduct(formValid, $scope.product.id, brandId, categoryId, code, establishmentId, image, name, remarks);
            } else {
                $scope.addProduct(formValid, brandId, categoryId, code, establishmentId, image, name, remarks);
            }
        }
        //Setup view (fired from mallistaSetup directive)
        $scope.setup = function () {
            //Recover data from previous view
            var item = ProductItem;
            if (item.valid == true) {
                $scope.product.id = item.id;
                $scope.product.brandId = item.brandId;
                $scope.product.categoryId = item.categoryId;
                $scope.product.creationDate = item.creationDate;
                $scope.product.code = item.code;
                $scope.product.establishmentId = item.establishmentId;
                $scope.product.image = item.image;
                $scope.product.isEdition = item.isEdition;
                $scope.product.modificationDate = item.modificationDate;
                $scope.product.name = item.name;
                $scope.product.remarks = item.remarks;
                item.valid = false;
            }
            //Load data from master tables
            getBrands();
            getCategories();
            getEstablishments();
        }
        //Get the corresponding translation for a token
        $scope.t = function (token) {
            return TranslatorService.getToken(token);
        }
        //Update an specific product
        $scope.updateProduct = function (formValid, id, brandId, categoryId, code, establishmentId, image, name, remarks) {
            //Validate form
            if (formValid == true) {
                $q.when(ProductHelper.updateProduct(id, brandId, categoryId, code, establishmentId, image, name, remarks), function () {
                    $location.path("/tab/products");
                }, function (err) {
                    ToolService.showMessage("UNEXPECTED_ERROR");
                });
            } else {
                ToolService.showMessage("INVALID_FORM");
            }
        }
        //Get the full url of an specific image
        $scope.urlForImage = function (imageName) {
            var trueOrigin = cordova.file.dataDirectory + imageName;
            return trueOrigin;
        }
    }])
//#endregion CREATE PRODUCT CONTROLLER

//#region SHOW PRODUCT CONTROLLER
    .controller('showProductCtrl', ['$scope', '$q', 'TranslatorService', 'ProductHelper', 'ProductItem', 'ToolService',
    function ($scope, $q, TranslatorService, ProductHelper, ProductItem, ToolService) {
        $scope.product = [];
        $scope.brandName = null;
        $scope.categoryName = null;
        $scope.establishmentName = null;

        //Get the brand name
        getBrandName = function () {
            $q.when(ProductHelper.getBrandName($scope.product.brandId), function (res) {
                $scope.brandName = res;
            })
        }
        //Get the category name
        getCategoryName = function () {
            $q.when(ProductHelper.getCategoryName($scope.product.categoryId), function (res) {
                $scope.categoryName = res;
            })
        }
        //Get the establishment name
        getEstablishmentName = function () {
            $q.when(ProductHelper.getEstablishmentName($scope.product.establishmentId), function (res) {
                $scope.establishmentName = res;
            })
        }
        //Indicate whether a value is empty
        $scope.isEmpty = function (value) {
            return ToolService.isEmpty(value);
        }
        //Setup view (fired from mallistaSetup directive)
        $scope.setup = function () {
            //Recover data from previous view
            var item = ProductItem;
            if (item.valid == true) {
                $scope.product.id = item.id;
                $scope.product.brandId = item.brandId;
                $scope.product.categoryId = item.categoryId;
                $scope.product.creationDate = item.creationDate;
                $scope.product.code = item.code;
                $scope.product.establishmentId = item.establishmentId;
                $scope.product.image = item.image;
                $scope.product.isEdition = item.isEdition;
                $scope.product.modificationDate = item.modificationDate;
                $scope.product.name = item.name;
                $scope.product.remarks = item.remarks;
                item.valid = false;
            }
            //Load data from master tables
            getBrandName();
            getCategoryName();
            getEstablishmentName();
        }
        //Get the corresponding translation for a token
        $scope.t = function (token) {
            return TranslatorService.getToken(token);
        }
    }])
//#endregion SHOW PRODUCT CONTROLLER

//#region STATISTICS CONTROLLER
    .controller('statisticsCtrl', ['$rootScope', '$scope', '$state', '$q',
                '$location', 'TranslatorService', 'StatisticsService',
                'ToolService', 'ReportItem',
    function ($rootScope, $scope, $state, $q, $location, TranslatorService,
                StatisticsService, ToolService, ReportItem) {
        $scope.data = {};
        $scope.reportResult = [];
        $scope.report = [];
        //Datepicker for start date
        $scope.startDate = {
            titleLabel: TranslatorService.getToken("SELECT_DATE"),
            todayLabel: TranslatorService.getToken("TODAY"),
            closeLabel: TranslatorService.getToken("CLOSE"),
            setLabel: TranslatorService.getToken("SET"),
            inputDate: $scope.data.startDate,
            mondayFirst: true,
            weekDaysList: TranslatorService.getWeekDayList(),
            monthList: TranslatorService.getMonthList(),
            templateType: 'popup',
            showTodayButton: 'true',
            modalHeaderColor: 'bar-positive',
            modalFooterColor: 'bar-positive',
            from: new Date(2015, 0, 1),
            to: new Date(2025, 11, 31),
            callback: function (val) {
                if (typeof (val) != 'undefined') {
                    this.inputDate = val;
                }
            },
            dateFormat: 'dd/MM/yyyy',
            closeOnSelect: false,
        };
        //Datepicker for end date
        $scope.endDate = {
            titleLabel: TranslatorService.getToken("SELECT_DATE"),
            todayLabel: TranslatorService.getToken("TODAY"),
            closeLabel: TranslatorService.getToken("CLOSE"),
            setLabel: TranslatorService.getToken("SET"),
            inputDate: $scope.data.endDate,
            mondayFirst: true,
            weekDaysList: TranslatorService.getWeekDayList(),
            monthList: TranslatorService.getMonthList(),
            templateType: 'popup',
            showTodayButton: 'true',
            modalHeaderColor: 'bar-positive',
            modalFooterColor: 'bar-positive',
            from: new Date(2015, 0, 1),
            to: new Date(2025, 11, 31),
            callback: function (val) {
                if (typeof (val) != 'undefined') {
                    this.inputDate = val;
                }
            },
            dateFormat: 'dd/MM/yyyy',
            closeOnSelect: false,
        };

        //Generate selected report
        runReport = function (report, startDate, endDate) {
            $q.when(StatisticsService.runReport(report, startDate, endDate), function (res) {
                $scope.reportResult = res;
            }, function (err) {
                ToolService.showMessage("UNEXPECTED_ERROR");
            });
        }
        //Get amount
        $scope.getAmount = function (value) {
            if (!isNaN(value))
                return parseFloat(Math.round(value * 100) / 100) + "€";
            else
                return "";
        }
        //Get description
        $scope.getDescription = function (description) {
            var result = description;
            if ($scope.data.id == StatisticsService.report_2) {
                var year = description.substr(description.indexOf("/") + 1);
                var month = parseInt(description.substr(0, description.indexOf("/")));
                if (!isNaN(month))
                    result = TranslatorService.getMonthName(month) + " - " + year;
                else
                    result = TranslatorService.getToken(description);
            }
            if (result == null || result == "")
                result = TranslatorService.getToken("NONE");
            return result;
        }
        //Get percentage
        $scope.getPercentage = function (value) {
            if (!isNaN(value))
                return parseFloat(Math.round(value * 100) / 100) + "%";
            else
                return "";
        }
        //Get quantity
        $scope.getQuantity = function (value) {
            if (!isNaN(value))
                return parseFloat(Math.round(value * 100) / 100);
            else
                return "";
        }
        //Get selected range
        $scope.getRange = function () {
            var start = ToolService.formatDate($scope.data.startDate);
            var end = ToolService.formatDate($scope.data.endDate);
            return TranslatorService.getToken("REPORT_RANGE", [start, end]);
        }
        //Go to de previous main view
        $scope.goHome = function () {
            $state.go($rootScope.fromState.name);
        }
        //Refresh view (fired from mallistaRefresh directive)
        $scope.refresh = function () {
            runReport($scope.data.id, $scope.data.startDate, $scope.data.endDate);
        }
        //Store selected report
        $scope.selectReport = function (report) {
            var data = ReportItem;
            data.clear(true);
            data.id = report;
            data.valid = true;
            switch (report) {
                case StatisticsService.report_1:
                    data.name = "PURCHASED_PRODUCTS_PER_PERIOD";
                    break;
                case StatisticsService.report_2:
                    data.name = "TOTAL_AMOUNT_PER_PERIOD";
                    break;
                case StatisticsService.report_3:
                    data.name = "TOTAL_AMOUNT_PER_CATEGORY";
                    break;
                case StatisticsService.report_4:
                    data.name = "TOTAL_AMOUNT_PER_BRAND";
                    break;
                case StatisticsService.report_5:
                    data.name = "TOTAL_AMOUNT_PER_ESTABLISHMENT";
                    break;
            }
            $location.path("/statistics/reportrange");
        }
        //Set report range
        $scope.setRange = function (startDate, endDate) {
            ReportItem.setRange(startDate, endDate);
            $location.path("/statistics/reportresult");
        }
        //Setup view (fired from mallistaSetup directive)
        $scope.setup = function () {
            $scope.report.r1 = StatisticsService.report_1;
            $scope.report.r2 = StatisticsService.report_2;
            $scope.report.r3 = StatisticsService.report_3;
            $scope.report.r4 = StatisticsService.report_4;
            $scope.report.r5 = StatisticsService.report_5;

            var item = ReportItem;
            if (item.valid) {
                $scope.data.id = item.id;
                $scope.data.name = item.name;
                $scope.data.startDate = item.startDate;
                $scope.data.endDate = item.endDate;
                $scope.startDate.inputDate = item.startDate;
                $scope.endDate.inputDate = item.endDate;
            }
        }
        //Get the corresponding translation for a token
        $scope.t = function (token) {
            return TranslatorService.getToken(token);
        }
    }])
//#endregion STATISTICS CONTROLLER

//#region SETTINGS CONTROLLER
    .controller('settingsCtrl', ['$rootScope', '$scope', '$state', 'TranslatorService', 'SettingService', 'ToolService',
    function ($rootScope, $scope, $state, TranslatorService, SettingService, ToolService) {
        $scope.showPictureInPending = false;
        $scope.showPictureInProduct = false;
        $scope.languageList = TranslatorService.getLanguageList();

        //Get settings
        getSettings = function () {
            SettingService.getSetting(SettingService.pendingShowPicture, false).then(function (res) {
                $scope.showPictureInPending = ToolService.isTrue(res);
            });
            SettingService.getSetting(SettingService.productShowPicture, false).then(function (res) {
                $scope.showPictureInProduct = ToolService.isTrue(res);
            });
        }
        //Go to de previous main view
        $scope.goHome = function () {
            $state.go($rootScope.fromState.name);
        }
        //Refresh data
        $scope.refresh = function () {
            getSettings();
        }
        //Save selected language
        $scope.setCurrentLanguage = function (lan) {
            TranslatorService.setCurrentLanguage(lan);
            lan = TranslatorService.getCurrentLanguage();
            SettingService.setSetting(SettingService.languageKey, lan);
        }
        //Save setting value (show pictures in pending product list)
        $scope.setShowPictureInPending = function (value) {
            SettingService.setSetting(SettingService.pendingShowPicture, value);
        }
        //Save setting value (show pictures in product list)
        $scope.setShowPictureInProduct = function (value) {
            SettingService.setSetting(SettingService.productShowPicture, value);
        }
        //Get the corresponding translation for a token
        $scope.t = function (token) {
            return TranslatorService.getToken(token);
        }
    }])
//#endregion SETTINGS CONTROLLER

//#region MAINTENANCE CONTROLLER
    .controller('maintenanceCtrl', ['$rootScope', '$scope', '$state', '$q', '$location', 'TranslatorService', 'ToolService', 'MaintenanceItem',
    function ($rootScope, $scope, $state, $q, $location, TranslatorService, ToolService, MaintenanceItem) {
        $scope.data = {};

        //Go to de previous main view
        $scope.goHome = function () {
            $state.go($rootScope.fromState.name);
        }
        //Store selected table
        $scope.openTable = function (table, description) {
            var data = MaintenanceItem;
            data.clear();
            data.tableDescription = TranslatorService.getToken(description);
            data.table = table;
            data.valid = true;
            $location.path("/maintenance/table");
        }
        //Get the corresponding translation for a token
        $scope.t = function (token) {
            return TranslatorService.getToken(token);
        }
    }])
//#endregion MAINTENANCE CONTROLLER

//#region TABLE CONTROLLER
    .controller('maintenanceTableCtrl', ['$scope', '$state', '$q', '$location', '$ionicActionSheet', '$ionicPopup',
                                        'TranslatorService', 'ToolService', 'MaintenanceHelper', 'MaintenanceItem',
                                        'ProductHelper', 'ShoppingListHelper', 'ImageService',
    function ($scope, $state, $q, $location, $ionicActionSheet, $ionicPopup,
                TranslatorService, ToolService, MaintenanceHelper, MaintenanceItem,
                ProductHelper, ShoppingListHelper, ImageService) {
        $scope.data = {};
        $scope.searcherInput = "";
        $scope.registerList = [];
        $scope.selectedItem = {};
        $scope.brandList = [];
        $scope.categoryList = [];
        $scope.establishmentList = [];
        $scope.productList = [];
        $scope.shoppingLists = [];

    //#region Private methods
        //Get the brand list 
        getBrandList = function () {
            ProductHelper.getFullBrandList(true).then(function (res) {
                $scope.brandList = res;
            }, function (err) {
                $scope.brandList = [];
            });
        }
        //Get the brand name
        getBrandName = function (brandId) {
            $q.when(ProductHelper.getBrandName(brandId), function (res) {
                $scope.data.brandName = res;
            })
        }
        //Get the category list
        getCategoryList = function () {
            ProductHelper.getFullCategoryList(true).then(function (res) {
                $scope.categoryList = res;
            }, function (err) {
                $scope.categoryList = [];
            });
        }
        //Get the category name
        getCategoryName = function (categoryId) {
            $q.when(ProductHelper.getCategoryName(categoryId), function (res) {
                $scope.data.categoryName = res;
            })
        }
        //Get the establishment list 
        getEstablishmentList = function () {
            ProductHelper.getFullEstablishmentList(true).then(function (res) {
                $scope.establishmentList = res;
            }, function (err) {
                $scope.establishmentList = [];
            });
        }
        //Get the establishment name
        getEstablishmentName = function (establishmentId) {
            $q.when(ProductHelper.getEstablishmentName(establishmentId), function (res) {
                $scope.data.establishmentName = res;
            })
        }
        //Get the product list 
        getProductList = function () {
            ProductHelper.getFullProductList().then(function (res) {
                $scope.productList = res;
            }, function (err) {
                $scope.productList = [];
            });
        }
        //Get the product name
        getProductName = function (productId) {
            $q.when(ProductHelper.getProductName(productId), function (res) {
                $scope.data.productName = res;
            })
        }
        //Get the list of shopping list 
        getShoppingLists = function () {
            ShoppingListHelper.getFullShoppingLists().then(function (res) {
                $scope.shoppingLists = res;
            }, function (err) {
                $scope.shoppingLists = [];
            });
        }
        //Get the shopping list name
        getShoppingListName = function (shoppingListId) {
            $q.when(ShoppingListHelper.getShoppingListName(shoppingListId), function (res) {
                $scope.data.shoppingListName = res;
            })
        }
    //#endregion Private methods

        //Add an image
        $scope.addImage = function (type) {
            $q.when(ImageService.addImage(type), function (name) {
                if (name != null && name != undefined && name != "")
                    $scope.data.image = name;
            });
        }
        //Add a new shopping list
        $scope.addRegister = function () {
            var data = MaintenanceItem;
            data.clear();
            data.table = $scope.data.table;
            data.tableDescription = $scope.data.tableDescription;
            data.fieldList = MaintenanceHelper.getTableFieldList($scope.data.table);
            data.activated = 1;
            data.isEdition = true;
            data.valid = true;
            $location.path("/maintenance/createregister");
        }
        //Clean the searcher input control
        $scope.cleanSearcherInput = function () {
            $scope.searcherInput = "";
        }
        //Show a popup in order to confirm deletion
        $scope.confirmDeleteRegister = function (item) {
            if (!$scope.tableCanDelete()) {
                ToolService.showMessage('UNABLE_TO_DELETE');
                return;
            }
            if (ToolService.isEmpty(item.id)) {
                ToolService.showMessage("NO_ROW_SELECTED");
                return;
            }
            var confirmPopup = $ionicPopup.confirm({
                title: TranslatorService.getToken('DELETE_REGISTER_TITLE'),
                template: TranslatorService.getToken('DELETE_REGISTER_BODY', item.name)
            });
            confirmPopup.then(function (res) {
                if (res) {
                    $scope.deleteRegister(item.id);
                }
            });
        }
        //Create a new register
        $scope.createRegister = function (formValid, item) {
            //Validate form
            if (formValid == true) {
                $q.when(MaintenanceHelper.addRegister($scope.data.table, item), function () {
                    $location.path("/maintenance/table");
                }, function (err) {
                    ToolService.showMessage("UNEXPECTED_ERROR");
                });
            } else {
                ToolService.showMessage("INVALID_FORM");
            }
        }
        //Delete the selected item
        $scope.deleteRegister = function (id) {
            MaintenanceHelper.deleteRegister($scope.data.table, id).then(function () {
                $scope.refresh();
            });
        }
        //Edit the selected item
        $scope.editRegister = function (item, refreshItem) {
            $scope.showRegister(item, true, refreshItem);
        }
        //Get amount
        $scope.getAmount = function (value) {
            if (!isNaN(value))
                return ToolService.round(value, 2) + "€";
            else
                return "";
        }
        //Get either a real image or a dummy image
        $scope.getImage = function (url) {
            return ToolService.getImage(url);
        }
        //Indicate whether a value is empty
        $scope.isEmpty = function (value) {
            return ToolService.isEmpty(value);
        }
        //Indicate whether a value is true
        $scope.isTrue = function (value) {
            return ToolService.isTrue(value);
        }
        //Refresh data (fired from mallistaRefresh directive)
        $scope.refresh = function () {
            var data = MaintenanceItem;
            $scope.data.table = data.table;
            $scope.data.tableDescription = data.tableDescription;
            data.fieldList = MaintenanceHelper.getTableFieldList(data.table);
            $scope.data.fieldList = data.fieldList;
            MaintenanceHelper.getRegisterList($scope.data.table).then(function (res) {
                $scope.registerList = res;
            }, function (err) {
                $scope.registerList = [];
            });
        }
        //Return a refreshed item
        $scope.refreshItem = function (item) {
            var result = null;
            angular.forEach($scope.registerList, function (register) {
                if (register.id == item.id) {
                    result = register;
                }
            });
            return result;
        }
        //Remove the image associated with register
        $scope.removeImage = function (type) {
            $scope.data.image = "";
        }
        //Save a register
        $scope.saveRegister = function (formValid, item){
            if (item.id != undefined && item.id > 0) {
                $scope.updateRegister(formValid, item);
            } else {
                $scope.createRegister(formValid, item);
            }
        }
        //Store the selected item
        $scope.selectItem = function (item) {
            $scope.selectedItem = item;
        }
        //Show de context menu for selected item
        $scope.showContextMenu = function (item) {
            $scope.selectedItem = item;
            // Show the action sheet
            if ($scope.tableCanDelete()) {
                //...for table Purchases
                var hideSheet = $ionicActionSheet.show({
                    buttons: [
                      { text: TranslatorService.getToken("VIEW_DETAILS") },
                      { text: TranslatorService.getToken("EDIT") },
                      { text: TranslatorService.getToken("DELETE") },
                    ],
                    titleText: '',
                    cssClass: 'mallista-action-sheet',
                    buttonClicked: function (index) {
                        if (index == 0) {
                            $scope.showRegister(item, false);
                        } else if (index == 1) {
                            $scope.editRegister(item);
                        } else if (index == 2) {
                            $scope.confirmDeleteRegister(item);
                        }
                        return true;
                    }
                });
            } else {
                //...for the rest of tables
                var hideSheet = $ionicActionSheet.show({
                    buttons: [
                      { text: TranslatorService.getToken("VIEW_DETAILS") },
                      { text: TranslatorService.getToken("EDIT") },
                    ],
                    titleText: '',
                    cssClass: 'mallista-action-sheet',
                    buttonClicked: function (index) {
                        if (index == 0) {
                            $scope.showRegister(item, false);
                        } else if (index == 1) {
                            $scope.editRegister(item);
                        }
                        return true;
                    }
                });
            }
        }
        //Show selected item
        $scope.showRegister = function (item, edit, refreshItem) {
            if (ToolService.isEmpty(item.id)) {
                ToolService.showMessage("NO_ROW_SELECTED");
                return;
            }
            if (refreshItem == true)
                item = $scope.refreshItem(item);

            var data = MaintenanceItem;
            data.clear();
            data.tableDescription = $scope.data.tableDescription;
            data.table = $scope.data.table;
            data.fieldList = MaintenanceHelper.getTableFieldList(data.table);
            data.id = item.id;
            data.activated = item.activated;
            data.creationDate = item.creationDate;
            data.pDate = item.pDate;
            data.isEdition = edit == true;
            data.description = item.description;
            data.remarks = item.remarks;
            data.brandId = item.brandId;
            data.categoryId = item.categoryId;
            data.establishmentId = item.establishmentId;
            data.productId = item.productId;
            data.shoppingListId = item.shoppingListId;
            data.purchaseDate = item.purchaseDate;
            data.purchasedQty = item.purchasedQty;
            data.unitPrice = item.unitPrice;
            data.totalAmount = item.totalAmount;
            data.name = item.name;
            data.valid = true;
            data.creationTime = item.creationTime;
            data.modificationDate = item.modificationDate;
            data.modificationTime = item.modificationTime;
            data.requestedQty = item.requestedQty;
            data.cDate = item.cDate;
            data.mDate = item.mDate;
            data.image = item.image;
            data.latitude = item.latitude;
            data.longitude = item.longitude;
            data.code = item.code;
            data.clau = item.clau;
            data.valor = item.valor;
            if (edit == true)
                $location.path("/maintenance/createregister");
            else
                $location.path("/maintenance/showregister");
        }
        //Setup view (fired from mallistaSetup directive)
        $scope.setup = function () {
            var data = MaintenanceItem;
            if (data.valid) {
                $scope.data.table = data.table;
                $scope.data.tableDescription = data.tableDescription;
                data.fieldList = MaintenanceHelper.getTableFieldList(data.table);
                $scope.data.id = data.id;
                $scope.data.activated = data.activated;
                $scope.data.isActivated = ToolService.isTrue(data.activated);
                $scope.data.creationDate = data.creationDate;
                $scope.data.pDate = data.pDate;
                $scope.data.isEdition = data.isEdition;
                $scope.data.description = data.description;
                $scope.data.remarks = data.remarks;
                $scope.data.brandId = data.brandId;
                $scope.data.categoryId = data.categoryId;
                $scope.data.establishmentId = data.establishmentId;
                $scope.data.productId = data.productId;
                $scope.data.shoppingListId = data.shoppingListId;
                $scope.data.purchaseDate = data.purchaseDate;
                $scope.data.purchasedQty = data.purchasedQty;
                $scope.data.unitPrice = data.unitPrice;
                $scope.data.totalAmount = data.totalAmount;
                $scope.data.name = data.name;
                $scope.data.modificationDate = data.modificationDate;
                $scope.data.modificationTime = data.modificationTime;
                $scope.data.requestedQty = data.requestedQty;
                $scope.data.cDate = data.cDate;
                $scope.data.mDate = data.mDate;
                $scope.data.image = data.image;
                $scope.data.latitude = data.latitude;
                $scope.data.longitude = data.longitude;
                $scope.data.fieldList = data.fieldList;
                $scope.data.code = data.code;
                $scope.data.clau = data.clau;
                $scope.data.valor = data.valor;
                data.valid = false;
            }
            //Load brand name or brand list
            if ($scope.tableHasField("brandId")) {
                if (data.isEdition)
                    getBrandList();
                else if (data.brandId != undefined)
                    getBrandName(data.brandId);
            }
            //Load category name or category list
            if ($scope.tableHasField("categoryId")) {
                if (data.isEdition)
                    getCategoryList();
                else if (data.categoryId != undefined)
                    getCategoryName(data.categoryId);
            }
            //Load establishment name or establishment list
            if ($scope.tableHasField("establishmentId")) {
                if (data.isEdition)
                    getEstablishmentList();
                else if (data.establishmentId != undefined)
                    getEstablishmentName(data.establishmentId);
            }
            //Load product name or product list
            if ($scope.tableHasField("productId")) {
                if (data.isEdition)
                    getProductList();
                else if (data.productId != undefined)
                    getProductName(data.productId);
            }
            //Load shopping list name or shoppingLists
            if ($scope.tableHasField("shoppingListId")) {
                if (data.isEdition)
                    getShoppingLists();
                else if (data.shoppingListId != undefined)
                    getShoppingListName(data.shoppingListId);
            }
            $scope.refresh();
        }
        //Get the corresponding translation for a token
        $scope.t = function (token) {
            return TranslatorService.getToken(token);
        }
        //Indicate whether a table can delete registers
        $scope.tableCanDelete = function () {
            return MaintenanceHelper.tableCanDelete($scope.data.table);
        }
        //Indicate whether a table contains a specific field
        $scope.tableHasField = function (field) {
            var result = false;
            if ($scope.data.fieldList != undefined)
                result = $scope.data.fieldList.indexOf(field) > -1;
            return result;
        }
        //Update an specific register
        $scope.updateRegister = function (formValid, item) {
            //Validate form
            if (formValid == true) {
                $q.when(MaintenanceHelper.updateRegister($scope.data.table, item), function () {
                    $location.path("/maintenance/table");
                }, function (err) {
                    ToolService.showMessage("UNEXPECTED_ERROR");
                });
            } else {
                ToolService.showMessage("INVALID_FORM");
            }
        }
    }])
//#endregion TABLE CONTROLLER

//#region ABOUT CONTROLLER
    .controller('aboutCtrl', ['$rootScope', '$scope', '$state', 'ToolService', 'TranslatorService',
    function ($rootScope, $scope, $state, ToolService, TranslatorService) {
        $scope.data = {};

        //Go to de previous main view
        $scope.goHome = function () {
            $state.go($rootScope.fromState.name);
        }
        //Refresh data
        $scope.refresh = function () {
            $scope.data.version = ToolService.getAppVersion();
            $scope.data.line_1 = $scope.t('ABOUT_LINE_1');
            $scope.data.line_2 = $scope.t('ABOUT_LINE_2');
            $scope.data.line_3 = $scope.t('ABOUT_LINE_3');
            $scope.data.line_4 = $scope.t('ABOUT_LINE_4');
        }
        //Get the corresponding translation for a token
        $scope.t = function (token) {
            return TranslatorService.getToken(token);
        }
    }])
//#endregion SETTINGS CONTROLLER

; //end module
