﻿angular.module('mallista.database.service', [])
    .factory('mallistaDB', ['$cordovaSQLite', '$rootScope', '$q', '$ionicPlatform', '$ionicLoading', 'ToolService',
    function ($cordovaSQLite, $rootScope, $q, $ionicPlatform, $ionicLoading, ToolService) {
        var self = this;
        var dbName = "mallista.db";
        var _db = null;

        //ATTENTION -> Don't use on release mode
        //That's just for developing purposes
        var _checkDatabase = false;

        //ATTENTION -> Put true before deploy the application
        var _isDevice = true;

        //Check out whether a database's object  exists or not
        self._checkObject = function (name, type) {
            var q = $q.defer();
            var query = "SELECT name FROM sqlite_master WHERE type = ? AND name = ?;";
            self._executeSql(query, [type, name]).then(function (res){
                var result = false;
                if (res.rows.length > 0) {
                    result = res.rows.item(0).name;
                } else {
                    ToolService.showMessage("Name: " + name + "\nType: " + type + "\nExists: " + result)
                }
                q.resolve();
            }, function (err) {
                ToolService.showMessage(err);
                q.reject(err);
            });
            return q.promise
        }
        //Create all database's object
        self._createDatabase = function (){
            var queries = [
                "CREATE TABLE IF NOT EXISTS Brands (" +
                                                "brandId integer primary key" +
                                                ", activated numeric not null default 1" +
                                                ", name text not null);",
                "CREATE UNIQUE INDEX IF NOT EXISTS b_name ON Brands(name);",
                "CREATE TABLE IF NOT EXISTS Categories (" +
                                                "categoryId integer primary key" +
                                                ", activated numeric not null default 1" +
                                                ", name text not null);",
                "CREATE UNIQUE INDEX IF NOT EXISTS c_name ON Categories(name);",
                "CREATE TABLE IF NOT EXISTS Establishments (" +
                                                "establishmentId integer primary key" +
                                                ", activated numeric not null default 1" +
                                                ", image text" +
                                                ", latitude numeric" +
                                                ", longitude numeric" +
                                                ", name text not null" +
                                                ", remarks blob);",
                "CREATE UNIQUE INDEX IF NOT EXISTS e_name ON Establishments(name);",
                "CREATE TABLE IF NOT EXISTS Products (" +
                                                "productId integer primary key" +
                                                ", activated numeric not null default 1" +
                                                ", brandId integer" +
                                                ", categoryId integer" +
                                                ", code text" +
                                                ", creationDate numeric not null default CURRENT_TIMESTAMP" +
                                                ", establishmentId integer" +
                                                ", image text" +
                                                ", modificationDate numeric" +
                                                ", name text not null" +
                                                ", remarks blob" +
                                                ", FOREIGN KEY(brandId) REFERENCES Brands(brandId)" +
                                                " ON DELETE RESTRICT ON UPDATE CASCADE" +
                                                ", FOREIGN KEY(categoryId) REFERENCES Categories(categoryId)" +
                                                " ON DELETE RESTRICT ON UPDATE CASCADE" +
                                                ", FOREIGN KEY(establishmentId) REFERENCES Establishments(establishmentId)" +
                                                " ON DELETE RESTRICT ON UPDATE CASCADE" +
                                                ");",
                "CREATE UNIQUE INDEX IF NOT EXISTS p_name ON Products(name);",
                "CREATE TABLE IF NOT EXISTS ShoppingLists (" +
                                                "shoppingListId integer primary key" +
                                                ", activated numeric not null default 1" +
                                                ", creationDate numeric not null default CURRENT_TIMESTAMP" +
                                                ", modificationDate numeric" +
                                                ", name text not null" +
                                                ", remarks blob);",
                "CREATE UNIQUE INDEX IF NOT EXISTS sp_name ON ShoppingLists(name);",
                "CREATE TABLE IF NOT EXISTS Purchases (" +
                                                "purchaseId integer primary key" +
                                                ", shoppingListId integer not null" +
                                                ", productId integer not null" +
                                                ", brandId integer" +
                                                ", establishmentId integer" +
                                                ", purchaseDate numeric not null default CURRENT_TIMESTAMP" +
                                                ", purchasedQty numeric not null default 0" +
                                                ", totalAmount numeric not null default 0" +
                                                ", unitPrice numeric not null default 0" +
                                                ", FOREIGN KEY(shoppingListId) REFERENCES ShoppingLists(shoppingListId)" +
                                                " ON DELETE RESTRICT ON UPDATE CASCADE" +
                                                ", FOREIGN KEY(productId) REFERENCES Products(productId)" +
                                                " ON DELETE RESTRICT ON UPDATE CASCADE" +
                                                ", FOREIGN KEY(brandId) REFERENCES Brands(brandId)" +
                                                " ON DELETE RESTRICT ON UPDATE CASCADE" +
                                                ", FOREIGN KEY(establishmentId) REFERENCES Establishments(establishmentId)" +
                                                " ON DELETE RESTRICT ON UPDATE CASCADE" +
                                                ");",
                "CREATE TABLE IF NOT EXISTS Settings (" +
                                                "clau text primary key not null" +
                                                ", valor text);",
                "CREATE UNIQUE INDEX IF NOT EXISTS settings_key ON Settings(clau);",
                "CREATE TABLE IF NOT EXISTS ShoppingListProducts (" +
                                                "shoppingListId integer not null" +
                                                ", productId integer not null" +
                                                ", creationDate numeric not null default CURRENT_TIMESTAMP" +
                                                ", modificationDate numeric" +
                                                ", purchasedQty numeric not null default 0" +
                                                ", requestedQty numeric not null default 0" +
                                                ", FOREIGN KEY(shoppingListId) REFERENCES ShoppingLists(shoppingListId)" +
                                                " ON DELETE RESTRICT ON UPDATE CASCADE" +
                                                ", FOREIGN KEY(productId) REFERENCES Products(productId)" +
                                                " ON DELETE RESTRICT ON UPDATE CASCADE" +
                                                ");",
                "CREATE UNIQUE INDEX IF NOT EXISTS slp_shoppinglistproduct ON ShoppingListProducts(shoppingListId, productId);",
            ];

            var objectList = [
                ["Brands", "table"],
                ["b_name", "index"],
                ["Categories", "table"],
                ["c_name", "index"],
                ["Establishments", "table"],
                ["e_name", "index"],
                ["Products", "table"],
                ["p_name", "index"],
                ["ShoppingLists", "table"],
                ["sp_name", "index"],
                ["Purchases", "table"],
                ["Settings", "table"],
                ["settings_key", "index"],
                ["ShoppingListProducts", "table"],
                ["slp_shoppinglistproduct", "index"],
            ];
            var q = $q.defer();

            if (_checkDatabase == true) {
                 $q.when(self._deleteDatabase(), function () {
                    $q.when(self._executeMultiSql(queries), function () {
                        //That's just for developing purposes
                        for (var i = 0; i < objectList.length; i++) {
                            self._checkObject(objectList[i][0], objectList[i][1]);
                        }
                        var query = "SELECT COUNT(*) AS total FROM Categories";
                        $q.when(self._executeSql(query, []), function (res) {
                            var total = 0;
                            if (res.rows.length > 0) {
                                total = res.rows.item(0).total;
                            }
                            if (total == 0) {
                                $q.when(self._insertBrands(), function () {
                                    $q.when(self._insertCategories(), function () {
                                        $q.when(self._insertEstablishments(), function (){
                                            q.resolve();
                                        });
                                        q.promise;
                                    })
                                    q.promise;
                                });
                                q.promise;
                            }
                        });
                        q.promise;
                    });
                    q.promise;
                 });
                 q.promise;
            } else {
                $q.when(self._executeMultiSql(queries), function () {
                    var query = "SELECT COUNT(*) AS total FROM Categories;";
                    $q.when(self._executeSql(query, []), function (res) {
                        var total = 0;
                        if (res.rows.length > 0) {
                            total = res.rows.item(0).total;
                        }
                        if (total == 0) {
                            $q.when(self._insertBrands(), function () {
                                $q.when(self._insertCategories(), function () {
                                    $q.when(self._insertEstablishments(), function () {
                                        q.resolve();
                                    });
                                    q.promise;
                                })
                                q.promise;
                            });
                            q.promise;
                        }
                    });
                    q.promise;
                });
                q.promise;
            }
            q.promise;
        }
        //Delete the database
        self._deleteDatabase = function () {
            window.sqlitePlugin.deleteDatabase({ name: dbName },
                function (res) {
                    ToolService.showMessage("Database deleted!!!");
                },
                function (err) {
                    $ionicLoading.hide();
                    ToolService.showMessage(err.message);
                });
            return;
        }
        //Execute several queries into a transaction
        self._executeMultiSql = function (queries, params, silence) {
            var q = $q.defer();

            $ionicPlatform.ready(function () {
                self._getDb().transaction(function (tx) {
                    for (i = 0; i < queries.length; i++) {
                        if (params == undefined) {
                            tx.executeSql(queries[i]);
                        } else {
                            tx.executeSql(queries[i], params[i]);
                        }
                    }
                }, function (err) {
                    $ionicLoading.hide();
                    err = ToolService.getDBError(err);
                    if (silence != true && $rootScope.initializing == false)
                        ToolService.showMessage(err.message);
                    q.reject(err);
                }, function () {
                    q.resolve();
                });
            });
            return q.promise;
        }
        //Execute a query into a single-statement transaction
        self._executeSql = function (query, parameters, silence) {
            parameters = parameters || [];
            var q = $q.defer();

            $ionicPlatform.ready(function () {
                self._getDb().transaction(function (tx) {
                    tx.executeSql(query, parameters, function (tx, res) {
                        q.resolve(res);
                    });
                }, function (err) {
                    $ionicLoading.hide();
                    err = ToolService.getDBError(err);
                    if (silence != true && $rootScope.initializing == false)
                        ToolService.showMessage(err.message);
                    q.reject(err);
                }, function () {
                    q.resolve();
                });
            });
            return q.promise;
        }
        //Open and return a connection to the database
        self._getDb = function () {
            if (_db == null) {
                if (window.sqlitePlugin != undefined && self._isDevice()) {
                    //Open SQLite connection
                    _db = window.sqlitePlugin.openDatabase({ name: dbName });
                    //Activate referential integrity
                    _db.executeSql("PRAGMA foreign_keys = ON;");
                } else {
                    //Open Web SQL to debug in browser
                    _db = window.openDatabase(dbName, "1.0", dbName, 200000);
                }
            }
            return _db;
        }
        //Insert initial data for table "Brands"
        self._insertBrands = function () {
            var queries = [
                            "INSERT INTO Brands (name) VALUES ('Bimbo');",
                            "INSERT INTO Brands (name) VALUES ('Codorniu');",
                            "INSERT INTO Brands (name) VALUES ('Nestle');",
                            "INSERT INTO Brands (name) VALUES ('Panrico');",
                            "INSERT INTO Brands (name) VALUES ('Tarradelles');",
                            "INSERT INTO Brands (name) VALUES ('Viladrau');",
            ];
            return $q.when(self._executeMultiSql(queries));
        }
        //Insert initial data for table "Categories"
        self._insertCategories = function () {
            var queries = [
                            "INSERT INTO Categories (name) VALUES ('Begudes');",
                            "INSERT INTO Categories (name) VALUES ('Congelats');",
                            "INSERT INTO Categories (name) VALUES ('Drogueria');",
                            "INSERT INTO Categories (name) VALUES ('Embutits');",
                            "INSERT INTO Categories (name) VALUES ('Fruites');",
                            "INSERT INTO Categories (name) VALUES ('Làctics');",
                            "INSERT INTO Categories (name) VALUES ('Neteja');",
                            "INSERT INTO Categories (name) VALUES ('Refrigerats');",
            ];
            return $q.when(self._executeMultiSql(queries));
        }
        //Insert initial data for table "Establishments"
        self._insertEstablishments = function () {
            var queries = [
                            "INSERT INTO Establishments (name) VALUES ('Alcampo');",
                            "INSERT INTO Establishments (name) VALUES ('Aldi');",
                            "INSERT INTO Establishments (name) VALUES ('Carrefour');",
                            "INSERT INTO Establishments (name) VALUES ('Dia');",
                            "INSERT INTO Establishments (name) VALUES ('Hipercor');",
                            "INSERT INTO Establishments (name) VALUES ('Mercadona');",
            ];
            return $q.when(self._executeMultiSql(queries));
        }
        //Indicate whether the current platform is a device
        self._isDevice = function () {
            return _isDevice;
        }
        //Service's public methods
        return {
            //Create the database schema
            createDatabase: function () {
                return self._createDatabase();
            },
            //Execute a query into a single-statement transaction
            executeSql: function (query, parameters, silence) {
                return self._executeSql(query, parameters, silence);
            },
            //Execute several queries into a transaction
            executeMultiSql: function (queries, params, silence) {
                self._executeMultiSql(queries, params, silence);
            },
            //Indicate whether the application is configured to deploy on device
            isDevice: _isDevice,
        }
    }]);//end module
