﻿angular.module('mallista.image.service', [])
.factory('ImageService', ['$cordovaCamera', '$q', '$cordovaFileTransfer', 'ToolService',
function ($cordovaCamera, $q, $cordovaFileTransfer, ToolService) {
    var self = this;

    //Add an image into the directory "data" and return its url
    self._addImage = function (sourceType) {
        var q = $q.defer();

        var options = self._pictureOptions(sourceType);

        $cordovaCamera.getPicture(options).then(
            function (fileUri) {
                $cordovaFileTransfer.download(fileUri, cordova.file.dataDirectory + self._getImageName() + ".jpg", {}, true).then(
                    function (fileEntry) {
                        return q.resolve(fileEntry.nativeURL);
                    },
                    function (error) {
                        //We can not consider an error, when user
                        //cancel the action
                        if (error.indexOf("cancelled") < 0) {
                            ToolService.showMessage(error);
                        }
                        q.reject(null);
                    }
                );
            },
            function (err) {
                //We can not consider an error, when user
                //cancel the action
                if (err.indexOf("cancelled") < 0) {
                    ToolService.showMessage(err);
                }
                q.reject(null);
            }
        );
        return q.promise;
    }
    //Get a random name to assign to the image
    self._getImageName = function () {
        var text = "";
        var allowedCharList = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
        var d = new Date();
        var m = d.getTime();
 
        for (var i = 0; i < 10; i++) {
            text += allowedCharList.charAt(Math.floor(Math.random() * allowedCharList.length));
        }
        return m + "_" + text;
    }
     //Get the collection of options for pictures
    self._pictureOptions = function (sourceType) {
        var source;
        switch (sourceType) {
            case 0:
                source = Camera.PictureSourceType.CAMERA;
                break;
            case 1:
                source = Camera.PictureSourceType.PHOTOLIBRARY;
                break;
        }
        return {
            destinationType: Camera.DestinationType.FILE_URI,
            sourceType: source,
            allowEdit: false,
            encodingType: Camera.EncodingType.JPEG,
            popoverOptions: CameraPopoverOptions,
            saveToPhotoAlbum: false
        };
    }
    //Service's public methods
    return {
        addImage: function (sourceType) {
            return self._addImage(sourceType)
        }
    }
}]); //end module