﻿angular.module('mallista.setting.service', [])
    .factory('SettingService', ['$q', 'mallistaDB', function ($q, mallistaDB) {
        var self = this;
        var LANGUAGE_KEY = "LANGUAGE_KEY";
        var PENDING_SHOW_PICTURE = "PENDING_SHOW_PICTURE";
        var PRODUCT_SHOW_PICTURE = "PRODUCT_SHOW_PICTURE";
        var settings = [];

        //Get a setting value from local copy
        self._getLocalSetting = function (key) {
            var result = null;
            angular.forEach(settings, function (item) {
                if (item.key == key) {
                    result = item.value;
                }
            });
            return result;
        }
        //Get a setting value
        self._getSetting = function (key, defaultValue) {
            var q = $q.defer();
            var result = self._getLocalSetting(key);

            //If value is into local copy, we won't find on database
            if (result != null) {
                q.resolve(result);
            } else {
                var query = "SELECT valor FROM Settings WHERE clau = ?;"
                $q.when(mallistaDB.executeSql(query, [key]), function (res) {
                    if (res.rows.length > 0) {
                        result = res.rows.item(0).valor;
                    } else {
                        if (defaultValue != undefined) {
                            result = defaultValue;
                        }
                    }
                    self._setLocalSetting(key, result);
                    q.resolve(result);
                });
            }
            return q.promise;
        }
        //Indicate whether a key exists on database
        self._keyExists = function (key) {
            q = $q.defer();
            var result = false;
            var query = "SELECT COUNT(*) AS amount FROM Settings WHERE clau = ?;"
            mallistaDB.executeSql(query, [key]).then(function (res) {
                if (res.rows.length > 0) {
                    result = res.rows.item(0).amount > 0;
                }
                q.resolve(result);
            });
            return q.promise;
        }
        //Store a setting value into local copy
        self._setLocalSetting = function (key, value) {
            var setting = { key: key, value: value };
            var found = false;
            angular.forEach(settings, function (item, index) {
                if (item.key == key) {
                    settings[index] = setting;
                    found = true;
                }
            });
            if (!found) {
                settings.push(setting);
            }
        }
        //Save a setting value
        self._setSetting = function (key, value) {
            var query = null;
            self._keyExists(key).then(function (exists) {
                if (exists == true) {
                    query = "UPDATE Settings SET valor = ? WHERE clau = ?;";
                } else {
                    query = "INSERT INTO Settings (valor, clau) VALUES (?, ?);";
                }
                mallistaDB.executeSql(query, [value, key]);
            });
            self._setLocalSetting(key, value);
        }
        //Service's public methods and properties
        return {
            getSetting: function (key, defaultValue) {
                return self._getSetting(key, defaultValue);
            },
            setSetting: function (key, value) {
                self._setSetting(key, value);
            },
            languageKey: LANGUAGE_KEY,
            pendingShowPicture: PENDING_SHOW_PICTURE,
            productShowPicture: PRODUCT_SHOW_PICTURE,
        };
    }]); //end module
