﻿angular.module('mallista.statistics.service', [])
    .factory('StatisticsService', ['$q', '$ionicLoading', 'mallistaDB', 'TranslatorService', 'ToolService',
    function ($q, $ionicLoading, mallistaDB, TranslatorService, ToolService) {
        var self = this;
        var r0 = 0;
        var r1 = 1;
        var r2 = 2;
        var r3 = 3;
        var r4 = 4;
        var r5 = 5;

        //Format a date as a ISO date
        self._getISODate = function (date){
            var result = date.toISOString().substr(0, 8);
            var day = date.getDate();
            if (day < 10) {
                day = "0" + day;
            }
            return result + day;
        }
        //Convert a date to a string, valid to work with SQLite
        self._getSQLiteDate = function (date, addDay) {
            var result = new Date(date.toString());
            //result.setDate(result.getDate() + 0);
            if (addDay != undefined && !isNaN(addDay))
                result.setDate(result.getDate() + addDay);
            return self._getISODate(result); // result.toISOString().substr(0, 10);
        }
        //Launch the corresponding report
        self._runReport = function (report, startDate, endDate) {
            switch (report){
                case r1:
                    return self._runReport1(startDate, endDate);
                    break;
                case r2:
                    return self._runReport2(startDate, endDate);
                    break;
                case r3:
                    return self._runReport3(startDate, endDate);
                    break;
                case r4:
                    return self._runReport4(startDate, endDate);
                    break;
                case r5:
                    return self._runReport5(startDate, endDate);
                    break;
            }
        }
        //Generate the report 1
        self._runReport1 = function (startDate, endDate) {
            var q = $q.defer();
            var result = [];
            var startRange = self._getSQLiteDate(startDate);
            var endRange = self._getSQLiteDate(endDate);
            var totalAmount = 0;
            var query = "SELECT SUM(totalAmount) AS totalAmount" +
                        " FROM Purchases" +
                        " WHERE date(purchaseDate) BETWEEN date(?) AND date(?);";

            ToolService.showLoading("GENERATING_REPORT");
            $q.when(mallistaDB.executeSql(query, [startRange, endRange]), function (res) {
                if (res.rows.length > 0) {
                    for (var i = 0; i < res.rows.length; i++) {
                        var item = res.rows.item(i);
                        totalAmount = res.rows.item(0).totalAmount;
                    }
                }

                query = "SELECT SUM(p.totalAmount) AS totalAmount," +
                                " pd.name AS description," +
                                " SUM(p.purchasedQty) AS quantity," +
                                " (SUM(p.totalAmount) * 100 / ?) AS percentage" +
                        " FROM Purchases p" +
                            " LEFT JOIN Products pd" +
                            " ON p.productId = pd.productId" +
                        " WHERE date(p.purchaseDate) BETWEEN date(?) AND date(?)" +
                        " GROUP BY pd.name" +
                        " ORDER BY pd.name COLLATE NOCASE ASC;";

                ToolService.showLoading("GENERATING_REPORT");
                $q.when(mallistaDB.executeSql(query, [totalAmount, startRange, endRange]), function (res) {
                    for (var i = 0; i < res.rows.length; i++) {
                        var item = res.rows.item(i);
                        result.push(item);
                    }
                    $ionicLoading.hide();
                    q.resolve(result);
                }, function (err) {
                    $ionicLoading.hide();
                    q.reject(err);
                });
                return q.promise;
            });
            return q.promise;
        }
        //Generate the report 2
        self._runReport2 = function (startDate, endDate) {
            var q = $q.defer();
            var result = [];
            var startRange = self._getSQLiteDate(startDate);
            var endRange = self._getSQLiteDate(endDate);
            var totalAmount = 0;
            var query = "SELECT SUM(totalAmount) AS totalAmount," +
                                " 100 AS percentage," +
                                " 'TOTAL_PERIOD' AS description," +
                                " '0' AS position" +
                        " FROM Purchases" +
                        " WHERE date(purchaseDate) BETWEEN date(?) AND date(?);";

            ToolService.showLoading("GENERATING_REPORT");
            $q.when(mallistaDB.executeSql(query, [startRange, endRange]), function (res) {
                if (res.rows.length > 0){
                    for (var i = 0; i < res.rows.length; i++) {
                        var item = res.rows.item(i);
                        totalAmount = res.rows.item(0).totalAmount;
                        result.push(item);
                    }
                }

                query = "SELECT SUM(totalAmount) AS totalAmount," +
                                " strftime('%m/%Y', purchaseDate) AS description," +
                                " (SUM(totalAmount) * 100 / ?) AS percentage," +
                                " strftime('%Y%m', purchaseDate) AS position" +
                        " FROM Purchases" +
                        " WHERE date(purchaseDate) BETWEEN date(?) AND date(?)" +
                        " GROUP BY strftime('%m-%Y', purchaseDate)" +
                        " ORDER BY strftime('%Y%m', purchaseDate) ASC;";

                ToolService.showLoading("GENERATING_REPORT");
                $q.when(mallistaDB.executeSql(query, [totalAmount, startRange, endRange]), function (res) {
                    for (var i = 0; i < res.rows.length; i++) {
                        var item = res.rows.item(i);
                        result.push(item);
                    }
                    $ionicLoading.hide();
                    q.resolve(result);
                }, function (err) {
                    $ionicLoading.hide();
                    q.reject(err);
                });
                return q.promise;
            });
            return q.promise;
        }
        //Generate the report 3
        self._runReport3 = function (startDate, endDate) {
            var q = $q.defer();
            var result = [];
            var startRange = self._getSQLiteDate(startDate);
            var endRange = self._getSQLiteDate(endDate);
            var totalAmount = 0;
            var query = "SELECT SUM(totalAmount) AS totalAmount" +
                        " FROM Purchases" +
                        " WHERE date(purchaseDate) BETWEEN date(?) AND date(?);";

            ToolService.showLoading("GENERATING_REPORT");
            $q.when(mallistaDB.executeSql(query, [startRange, endRange]), function (res) {
                if (res.rows.length > 0) {
                    for (var i = 0; i < res.rows.length; i++) {
                        var item = res.rows.item(i);
                        totalAmount = res.rows.item(0).totalAmount;
                    }
                }

                query = "SELECT SUM(p.totalAmount) AS totalAmount," +
                                " c.name AS description," +
                                " SUM(p.purchasedQty) AS quantity," +
                                " (SUM(p.totalAmount) * 100 / ?) AS percentage" +
                        " FROM Purchases p" +
                            " LEFT JOIN Products pd" +
                            " ON p.productId = pd.productId" +
                            " LEFT JOIN Categories c" +
                            " ON c.categoryId = pd.categoryId" +
                        " WHERE date(p.purchaseDate) BETWEEN date(?) AND date(?)" +
                        " GROUP BY c.name" +
                        " ORDER BY c.name COLLATE NOCASE ASC;";

                ToolService.showLoading("GENERATING_REPORT");
                $q.when(mallistaDB.executeSql(query, [totalAmount, startRange, endRange]), function (res) {
                    for (var i = 0; i < res.rows.length; i++) {
                        var item = res.rows.item(i);
                        result.push(item);
                    }
                    $ionicLoading.hide();
                    q.resolve(result);
                }, function (err) {
                    $ionicLoading.hide();
                    q.reject(err);
                });
                return q.promise;
            });
            return q.promise;
        }
        //Generate the report 4
        self._runReport4 = function (startDate, endDate) {
            var q = $q.defer();
            var result = [];
            var startRange = self._getSQLiteDate(startDate);
            var endRange = self._getSQLiteDate(endDate);
            var totalAmount = 0;
            var query = "SELECT SUM(totalAmount) AS totalAmount" +
                        " FROM Purchases" +
                        " WHERE date(purchaseDate) BETWEEN date(?) AND date(?);";

            ToolService.showLoading("GENERATING_REPORT");
            $q.when(mallistaDB.executeSql(query, [startRange, endRange]), function (res) {
                if (res.rows.length > 0) {
                    for (var i = 0; i < res.rows.length; i++) {
                        var item = res.rows.item(i);
                        totalAmount = res.rows.item(0).totalAmount;
                    }
                }

                query = "SELECT SUM(p.totalAmount) AS totalAmount," +
                                " b.name AS description," +
                                " SUM(p.purchasedQty) AS quantity," +
                                " (SUM(p.totalAmount) * 100 / ?) AS percentage" +
                        " FROM Purchases p" +
                            " LEFT JOIN Brands b" +
                            " ON b.brandId = p.brandId" +
                        " WHERE date(p.purchaseDate) BETWEEN date(?) AND date(?)" +
                        " GROUP BY b.name" +
                        " ORDER BY b.name COLLATE NOCASE ASC;";

                ToolService.showLoading("GENERATING_REPORT");
                $q.when(mallistaDB.executeSql(query, [totalAmount, startRange, endRange]), function (res) {
                    for (var i = 0; i < res.rows.length; i++) {
                        var item = res.rows.item(i);
                        result.push(item);
                    }
                    $ionicLoading.hide();
                    q.resolve(result);
                }, function (err) {
                    $ionicLoading.hide();
                    q.reject(err);
                });
                return q.promise;
            });
            return q.promise;
        }
        //Generate the report 5
        self._runReport5 = function (startDate, endDate) {
            var q = $q.defer();
            var result = [];
            var startRange = self._getSQLiteDate(startDate);
            var endRange = self._getSQLiteDate(endDate);
            var totalAmount = 0;
            var query = "SELECT SUM(totalAmount) AS totalAmount" +
                        " FROM Purchases" +
                        " WHERE date(purchaseDate) BETWEEN date(?) AND date(?);";

            ToolService.showLoading("GENERATING_REPORT");
            $q.when(mallistaDB.executeSql(query, [startRange, endRange]), function (res) {
                if (res.rows.length > 0) {
                    for (var i = 0; i < res.rows.length; i++) {
                        var item = res.rows.item(i);
                        totalAmount = res.rows.item(0).totalAmount;
                    }
                }

                query = "SELECT SUM(p.totalAmount) AS totalAmount," +
                                " e.name AS description," +
                                " SUM(p.purchasedQty) AS quantity," +
                                " (SUM(p.totalAmount) * 100 / ?) AS percentage" +
                        " FROM Purchases p" +
                            " LEFT JOIN Establishments e" +
                            " ON e.establishmentId = p.establishmentId" +
                        " WHERE date(p.purchaseDate) BETWEEN date(?) AND date(?)" +
                        " GROUP BY e.name" +
                        " ORDER BY e.name COLLATE NOCASE ASC;";

                ToolService.showLoading("GENERATING_REPORT");
                $q.when(mallistaDB.executeSql(query, [totalAmount, startRange, endRange]), function (res) {
                    for (var i = 0; i < res.rows.length; i++) {
                        var item = res.rows.item(i);
                        result.push(item);
                    }
                    $ionicLoading.hide();
                    q.resolve(result);
                }, function (err) {
                    $ionicLoading.hide();
                    q.reject(err);
                });
                return q.promise;
            });
            return q.promise;
        }
        //Service's public methods and properties
        return {
            runReport: function (report, startDate, endDate) {
                return self._runReport(report, startDate, endDate);
            },
            reportNone: r0,
            report_1: r1,
            report_2: r2,
            report_3: r3,
            report_4: r4,
            report_5: r5,
        };
    }]);
