﻿angular.module('mallista.tool.service', [])
    .factory('ToolService', ['$ionicPopup', '$ionicLoading', '$timeout', 'TranslatorService',
    function ($ionicPopup, $ionicLoading, $timeout, TranslatorService) {
        var self = this;
        var _mallistaVersion = "1.0.0";

        //Convert a boolean to its meaning as a number
        self._convertBooleanToNumber = function (value){
            var result = 0;

            if (self._isTrue(value))
                result = 1;

            return result;
        }
        //Format a text replacing {n} with the corresponding arg
        self._format = function (text, args) {
            if (!angular.isArray(args))
                args = [args];

            if (text != undefined && text != null) {
                return text.replace(/\{(\d+)\}/g, function (match, capture) {
                    return args[1 * capture];
                });
            }
            return "";
        }
        //Format a date according to "day/month/year" format
        self._formatDate = function (date){
            var result = "";
            var copyDate = new Date(date.toString());
            var day = copyDate.getDate();
            var month = copyDate.getMonth() + 1;
            var year = copyDate.getFullYear();

            if(day < 10)
                day = "0" + day;

            if(month < 10)
                month = "0" + month;
            result =  day + "/" + month + "/" + year;
            return result;
        }
        //Get the error occurred on database
        self._getDBError = function (error) {
            var result = {};
            if (error.message == undefined || error.code == undefined){
                result = {code: -1, message: TranslatorService.getToken("ERROR_DATABASE")};
            } else {
                result = { code: self._getDBErrorCode(error.message), message: self._getDBErrorMessage(error.message) }
            }
            return result;
        }
        //Get either a real image or a dummy image
        self._getImage = function (url) {
            if (url != undefined && url != null && url != "") {
                return url;
            } else {
                return "img/nophoto.png";
            }
        }
        //Get the database error code
        self._getDBErrorCode = function (error_message){
            var result = -1;
            if (error_message.lastIndexOf("error: ") >= 0) {
                result = parseInt(error_message.substr(error_message.lastIndexOf("error: ") + 7), 10);
            }
            return result;
        }
        //Get the original error message from SQLite error list
        self._getDBErrorMessage = function (error_message) {
            var message = error_message;
            var errCode = self._getDBErrorCode(error_message);
            if (errCode > -1) {
                switch (errCode) {
                    case 0:
                        message = "Successful result";
                        break;
                    case 1:
                        message = "SQL error or missing database";
                        break;
                    case 2:
                        message = "Internal logic error in SQLite";
                        break;
                    case 3:
                        message = "Access permission denied";
                        break;
                    case 4:
                        message = "Callback routine requested an abort";
                        break;
                    case 5:
                        message = "The database file is locked";
                        break;
                    case 6:
                        message = "A table in the database is locked";
                        break;
                    case 7:
                        message = "A malloc() failed";
                        break;
                    case 8:
                        message = "Attempt to write a readonly database";
                        break;
                    case 9:
                        message = "Operation terminated by sqlite3_interrupt()";
                        break;
                    case 10:
                        message = "Some kind of disk I/O error occurred";
                        break;
                    case 11:
                        message = "The database disk image is malformed";
                        break;
                    case 12:
                        message = "Unknown opcode in sqlite3_file_control()";
                        break;
                    case 13:
                        message = "Insertion failed because database is full";
                        break;
                    case 14:
                        message = "Unable to open the database file";
                        break;
                    case 15:
                        message = "Database lock protocol error";
                        break;
                    case 16:
                        message = "Database is empty";
                        break;
                    case 17:
                        message = "The database schema changed";
                        break;
                    case 18:
                        message = "String or BLOB exceeds size limit";
                        break;
                    case 19:
                        message = "Abort due to constraint violation";
                        break;
                    case 20:
                        message = "Data type mismatch";
                        break;
                    case 21:
                        message = "Library used incorrectly";
                        break;
                    case 22:
                        message = "Uses OS features not supported on host";
                        break;
                    case 23:
                        message = "Authorization denied";
                        break;
                    case 24:
                        message = "Auxiliary database format error";
                        break;
                    case 25:
                        message = "2nd parameter to sqlite3_bind out of range";
                        break;
                    case 26:
                        message = "File opened that is not a database file";
                        break;
                    case 27:
                        message = "Notifications from sqlite3_log()";
                        break;
                    case 28:
                        message = "Warnings from sqlite3_log()";
                        break;
                    case 100:
                        message = "sqlite3_step() has another row ready";
                        break;
                    case 101:
                        message = "sqlite3_step() has finished executing";
                        break;
                }
            }
            return message;
        }
        //Get either the parameter's value or null
        self._getValueOrNull = function (value) {
            var result = value;
            if (value <= 0 || value == "null" || value == "")
                result = null;
            return result;
        }
        //Get the application version
        self._getAppVersion = function () {
            return TranslatorService.getToken('APP_VERSION', _mallistaVersion);
        }
        //Insert an item at first position of an array
        self._insertAtFirstPosition = function (array, item) {
            array.splice(0, 0, item);
        }
        //Indicate whether a value is empty
        self._isEmpty = function (value) {
            return value == undefined || value == null || value == "";
        }
        //Indicate whether a variable is a boolean with true value
        self._isTrue = function (value){
            return value == "true" || value == true;
        }
        //Replace all occurrences of a string inside another one
        self._replace = function (text, substr, newSubStr) {
            if (text != undefined && text != null) {
                return text.replace(new RegExp(substr, "g"), newSubStr);
            } else
                return null;
        }
        //Round a number according with a specific amount of decimals
        self._round = function (number, decimals) {
            if (isNaN(number))
                number = 0;
            if (isNaN(decimals))
                decimals = 0;

            return parseFloat(Math.round(number * Math.pow(10, decimals)) / Math.pow(10,decimals));
        }
        //Show loading...
        self._showLoading = function (token) {
            $ionicLoading.show({
                                template: '<ion-spinner></ion-spinner>&nbsp;' +
                                    TranslatorService.getToken(token) +
                                    '...'
                            });
            //In order to prevent blocking the application,
            //the maximum time showing the loading message
            //will be 5 seconds.
            $timeout(function () {
                $ionicLoading.hide();
            }, 5000);
        }
        //Show a message in a modal box
        self._showMessage = function (message, params) {
            var text = TranslatorService.getToken(message, params);
            text = self._replace(text, "\n", "</br>");
            var myPopup = $ionicPopup.show({
                template: "<p>" + text + "</p>",
                title: TranslatorService.getToken("MESSAGE_TITLE"),
                buttons: [
                  {
                      text: TranslatorService.getToken("ACCEPT"),
                      type: 'button-dark',
                  },
                ]
            });
        }
        //Service's public methods
        return {
            convertBooleanToNumber: function (value){
                return self._convertBooleanToNumber(value);
            },
            format: function (text, args) {
                return self._format(text, args);
            },
            formatDate: function (date) {
                return self._formatDate(date);
            },
            getAppVersion: function (){
                return self._getAppVersion();
            },
            getDBError: function (error) {
                return self._getDBError(error);
            },
            getImage: function (url) {
                return self._getImage(url);
            },
            getValueOrNull: function(value){
                return self._getValueOrNull(value);
            },
            insertAtFirstPosition: function (array, item){
                self._insertAtFirstPosition(array, item);
            },
            isEmpty: function(value){
                return self._isEmpty(value);
            },
            isTrue: function (value) {
                return self._isTrue(value);
            },
            replace: function (text, substr, newSubStr) {
                return self._replace(text, target, newSubStr);
            },
            round: function (number, decimals){
                return self._round(number, decimals);
            },
            showLoading: function (token) {
                return self._showLoading(token);
            },
            showMessage: function (message, params) {
                return self._showMessage(message, params);
            },
        }
    }])

    .directive('mallistaHideTabs', function ($rootScope) {
        return {
            restrict: 'A',
            link: function ($scope, $item, $attributes) {
                $rootScope.mallistaHideTabs = "tabs-item-hide";
                $scope.$on('$ionicView.enter', function () {
                    $rootScope.mallistaHideTabs = "tabs-item-hide";
                });
                $scope.$on('$ionicView.leave', function () {
                    $rootScope.mallistaHideTabs = '';
                });
            }
        };
    })
    .directive('mallistaRefresh', function ($rootScope) {
        return {
            restrict: 'A',
            link: function ($scope, $item, $attributes) {
                $scope.$on('$ionicView.enter', function () {
                    $scope.refresh();
                });
            }
        };
    })
    .directive('mallistaSetup', function ($rootScope) {
        return {
            restrict: 'A',
            link: function ($scope, $item, $attributes) {
                $scope.$on('$ionicView.loaded', function () {
                    $scope.setup();
                });
            }
        };
    })
    .directive('mallistaState', function ($rootScope) {
        return {
            restrict: 'A',
            link: function ($scope, $item, $attributes) {
                $scope.$on('$stateChangeSuccess', function (event, toState, toParams, fromState, fromParams) {
                    $rootScope.fromState = fromState;
                });
            }
        };
    })
    //.directive('mallistaDblclick', function () {
    //    const maxInterval = 400;
    //    var beginEvent;
    //    var eventInProgress = false;

    //    return {
    //        restrict: 'A',
    //        link: function ($scope, $item, $attributes) {
    //            $item.bind('click', function (e) {
    //                if (!eventInProgress) {
    //                    beginEvent = (new Date()).getTime();
    //                    eventInProgress = true;

    //                    setTimeout(function () {
    //                        eventInProgress = false;
    //                    }, maxInterval);
    //                }
    //                else {
    //                    eventInProgress = false;

    //                    var time = (new Date()).getTime();
    //                    if (time - beginEvent <= maxInterval) {
    //                        $scope.$apply($attributes.mallistaDblclick);
    //                    }
    //                }
    //            });
    //        }
    //    };
    //})
;