﻿angular.module('pending.helper.service', [])

.factory('PendingHelper', ['$q', '$ionicLoading', 'mallistaDB', 'TranslatorService', 'ToolService',
    function ($q, $ionicLoading, mallistaDB, TranslatorService, ToolService) {
    var self = this;

    //Add a new pending product
    self._addPending = function (productId, shoppingListId, quantity) {
        var q = $q.defer();

        $q.when(self._productExistsInShoppingList(productId, shoppingListId), function (res) {
            if (res == true)
                var query = "UPDATE ShoppingListProducts SET requestedQty = requestedQty + ? WHERE productId = ? AND shoppingListId = ?;";
            else
                var query = "INSERT INTO ShoppingListProducts (requestedQty, productId, shoppingListId) VALUES (?, ?, ?);";

            ToolService.showLoading("SAVING_DATA");
            mallistaDB.executeSql(query, [quantity, productId, shoppingListId]).then(function (res) {
                $ionicLoading.hide();
                q.resolve();
            }, function (err) {
                $ionicLoading.hide();
                q.reject(err);
            });
            return q.promise;
        });
        return q.promise;
    }
    //Save buying information for a specific pending product
    self._buyPending = function (id, shoppingListId, productId, brandId, establishmentId, purchasedQty, unitPrice, totalAmount) {
        var q = $q.defer();
        if (totalAmount == undefined || totalAmount == null)
            totalAmount = parseFloat((purchasedQty * Math.round(unitPrice * 1000000)) / 1000000);

        ToolService.showLoading("SAVING_DATA");
        var queries = ["UPDATE ShoppingListProducts SET purchasedQty = purchasedQty + ?" +
                            " WHERE shoppingListId = ? AND productId = ? AND rowId = ?",
                        "INSERT INTO Purchases (shoppingListId, productId, brandId, establishmentId, purchasedQty, totalAmount, unitPrice) VALUES (?, ?, ?, ?, ?, ?, ?);"];

        brandId = ToolService.getValueOrNull(brandId);
        establishmentId = ToolService.getValueOrNull(establishmentId);

        var params = [[purchasedQty, shoppingListId, productId, id], [shoppingListId, productId, brandId, establishmentId, purchasedQty, totalAmount, unitPrice]];
        $q.when(mallistaDB.executeMultiSql(queries, params), function (res) {
            $ionicLoading.hide();
            q.resolve();
        }, function (err) {
            $ionicLoading.hide();
            q.reject(err);
        });
        return q.promise;
    }
    //Delete a specific pending product
    self._deletePending = function (id) {
        var query = "UPDATE ShoppingListProducts SET requestedQty = purchasedQty, modificationDate = datetime('now', 'localtime') WHERE rowId = ?;";
        var q = $q.defer();

        ToolService.showLoading("SAVING_DATA");
        mallistaDB.executeSql(query, [id]).then(function (res) {
            $ionicLoading.hide();
            q.resolve();
        }, function (err) {
            $ionicLoading.hide();
            q.reject(err);
        });
        return q.promise;
    }

    //Get all pending products
    self._getPendingList = function () {
        var query = "SELECT " +
                    " slp.rowid AS id," +
                    " slp.shoppingListId," +
                    " sl.name AS shoppingListName," +
                    " strftime('%d/%m/%Y %H:%M:%S', slp.creationDate) AS creationDate," +
                    " strftime('%d/%m/%Y %H:%M:%S', slp.modificationDate) AS modificationDate," +
                    " slp.purchasedQty," +
                    " (slp.requestedQty - slp.purchasedQty) AS requestedQty," +
                    " p.productId, p.brandId, p.categoryId, p.code," +
                    " p.establishmentId, p.image," +
                    " p.name AS productName," +
                    " p.remarks AS productRemarks," +
                    " sl.remarks AS shoppingListRemarks," +
                    " b.name AS brandName," +
                    " c.name AS categoryName," +
                    " e.name AS establishmentName" +
                    " FROM ShoppingListProducts slp" +
                    " LEFT JOIN Products p ON slp.productId = p.productId" +
                    " LEFT JOIN ShoppingLists sl ON sl.shoppingListId = slp.shoppingListId" +
                    " LEFT JOIN Brands b ON b.brandId = p.brandId" +
                    " LEFT JOIN Categories c ON c.categoryId = p.categoryId" +
                    " LEFT JOIN Establishments e ON e.establishmentId = p.establishmentId" +
                    " WHERE " +
                    " sl.activated = 1 AND p.activated = 1 AND (slp.requestedQty - slp.purchasedQty) > 0" +
                    " ORDER BY p.name COLLATE NOCASE ASC, sl.name COLLATE NOCASE ASC;";

        var q = $q.defer();
        var result = [];

        ToolService.showLoading("LOADING_DATA");
        mallistaDB.executeSql(query).then(function (res) {
            if (res.rows.length > 0) {
                for (var i = 0; i < res.rows.length; i++) {
                    var item = res.rows.item(i);
                    result.push(item);
                }
            }
            $ionicLoading.hide();
            q.resolve(result);
        }, function (err) {
            $ionicLoading.hide();
            q.reject(err);
        });
        return q.promise;
    }

    //Find whether a product exists in a shopping list
    self._productExistsInShoppingList = function (productId, shoppingListId, id) {
        var params = null;
        if (id == undefined) {
            var query = "SELECT COUNT(*) AS total FROM ShoppingListProducts WHERE productId = ? AND shoppingListId = ?;";
            params = [productId, shoppingListId];
        } else {
            var query = "SELECT COUNT(*) AS total FROM ShoppingListProducts WHERE productId = ? AND shoppingListId = ? AND rowId <> ?;";
            params = [productId, shoppingListId, id];
        }
        var result = false;
        var q = $q.defer();

        ToolService.showLoading("SAVING_DATA");
        mallistaDB.executeSql(query, params).then(function (res) {
            $ionicLoading.hide();
            if (res.rows.length > 0)
                result = res.rows.item(0).total > 0;
            q.resolve(result);
        }, function (err) {
            $ionicLoading.hide();
            q.reject(err);
        });
        return q.promise;
    }

    //Update a specific pending product
    self._updatePending = function (id, shoppingListId, productId, quantity) {
        var q = $q.defer();

        $q.when(self._productExistsInShoppingList(productId, shoppingListId, id), function (res) {
            var params = null;
            var queries = null;

            ToolService.showLoading("SAVING_DATA");
            if (res == true) {
                queries = ["UPDATE ShoppingListProducts SET shoppingListId = ?, requestedQty = requestedQty - purchasedQty + ?, modificationDate = datetime('now', 'localtime')" +
                                    " WHERE shoppingListId = ? AND productId = ? AND rowId <> ?;",
                                "UPDATE ShoppingListProducts SET requestedQty = purchasedQty, modificationDate = datetime('now', 'localtime')" +
                                    " WHERE productId = ? AND rowId = ?;"];
                params = [[shoppingListId, quantity, shoppingListId, productId, id], [productId, id]];
            } else {
                queries = ["UPDATE ShoppingListProducts SET shoppingListId = ?, requestedQty = purchasedQty + ?, modificationDate = datetime('now', 'localtime') WHERE rowId = ?;"];
                params =  [[shoppingListId, quantity, id]];
            }                
            $q.when(mallistaDB.executeMultiSql(queries, params), function (res) {
                $ionicLoading.hide();
                q.resolve();
            }, function (err) {
                $ionicLoading.hide();
                q.reject(err);
            });
            return q.promise;
        });
        return q.promise;
    }
    //Service's public methods
    return {
        addPending: function (productId, shoppingListId, quantity){
            return self._addPending(productId, shoppingListId, quantity);
        },
        buyPending: function (id, shoppingListId, productId, brandId, establishmentId, purchasedQty, unitPrice, totalAmount){
            return self._buyPending(id, shoppingListId, productId, brandId, establishmentId, purchasedQty, unitPrice, totalAmount);
        },
        deletePending: function (id){
            return self._deletePending(id);
        },
        getPendingList: function () {
            return self._getPendingList();
        },
        updatePending: function (id, shoppingListId, productId, quantity) {
            return self._updatePending(id, shoppingListId, productId, quantity);
        }
    };
}])

//Filter used with pending product list
.filter('pendingFilter', [function () {
    return function (items, listId) {
        var foundItems = [];

        if (listId != undefined) {
            angular.forEach(items, function (item) {
                if (item.shoppingListId == listId || listId == 0)
                    foundItems.push(item);
            });
        }
        return foundItems;
    };
}])
; //end module


