﻿angular.module('product.helper.service', [])

.factory('ProductHelper', ['$q', '$ionicLoading', 'mallistaDB', 'TranslatorService', 'ToolService',
    function ($q, $ionicLoading, mallistaDB, TranslatorService, ToolService) {
    var self = this;

    //Insert a new product
    self._addProduct = function (brandId, categoryId, code, establishmentId, image, name, remarks) {
        var q = $q.defer();
        var query = "INSERT INTO Products (brandId, categoryId, code, establishmentId, image, name, remarks) VALUES (?, ?, ?, ?, ?, ?, ?);";

        brandId = ToolService.getValueOrNull(brandId);
        categoryId = ToolService.getValueOrNull(categoryId);
        establishmentId = ToolService.getValueOrNull(establishmentId);

        ToolService.showLoading("SAVING_DATA");
        mallistaDB.executeSql(query, [brandId, categoryId, code, establishmentId, image, name, remarks]).then(function (res) {
            $ionicLoading.hide();
            q.resolve();
        }, function (err) {
            $ionicLoading.hide();
            q.reject(err);
        });
        return q.promise;
    }

    //Delete a specific product
    self._deleteProduct = function (id) {
        var q = $q.defer();
        var query = "UPDATE Products SET activated = 0, modificationDate = datetime('now', 'localtime') WHERE productId = ?;";

        ToolService.showLoading("DELETING_DATA");
        mallistaDB.executeSql(query, [id]).then(function (res) {
            $ionicLoading.hide();
            q.resolve();
        }, function (err) {
            $ionicLoading.hide();
            q.reject(err);
        });
        return q.promise;
    }

    //Get all activated brands
    self._getBrandList = function (noneItem, fullList) {
        var q = $q.defer();
        var query = "SELECT brandId AS id, name FROM Brands WHERE {0} ORDER BY name COLLATE NOCASE ASC;";
        var where = "activated = 1";
        var result = [];

        if (noneItem != undefined && noneItem == true) {
            var none = { id: null, name: TranslatorService.getToken("NONE") };
            result.push(none);
        }

        if (fullList != undefined && fullList == true)
            where = "1 = 1";

        query = TranslatorService.format(query, where);

        ToolService.showLoading("LOADING_DATA");
        mallistaDB.executeSql(query).then(function (res) {
            if (res.rows.length > 0) {
                for (var i = 0; i < res.rows.length; i++) {
                    var item = res.rows.item(i);
                    result.push(item);
                }
            }
            $ionicLoading.hide();
            q.resolve(result);
        }, function (err) {
            $ionicLoading.hide();
            q.reject(err);
        });
        return q.promise;
    }

    //Get a brand name from its identificator
    self._getBrandName = function (brandId) {
        var q = $q.defer();
        var query = "SELECT name FROM Brands WHERE brandId = ?;";
        var result = null;

        ToolService.showLoading("LOADING_DATA");
        mallistaDB.executeSql(query, [brandId]).then(function (res) {
            if (res.rows.length > 0) {
                    result = res.rows.item(0).name;
            }
            $ionicLoading.hide();
            q.resolve(result);
        }, function (err) {
            $ionicLoading.hide();
            q.reject(err);
        });
        return q.promise;
    }

    //Get all activated categories
    self._getCategoryList = function (noneItem, fullList) {
        var q = $q.defer();
        var query = "SELECT categoryId AS id, name FROM Categories WHERE {0} ORDER BY name COLLATE NOCASE ASC;";
        var where = "activated = 1";
        var result = [];

        if (noneItem != undefined && noneItem == true) {
            var none = { id: null, name: TranslatorService.getToken("NONE") };
            result.push(none);
        }

        if (fullList != undefined && fullList == true)
            where = "1 = 1";

        query = TranslatorService.format(query, where);

        ToolService.showLoading("LOADING_DATA");
        mallistaDB.executeSql(query).then(function (res) {
            if (res.rows.length > 0) {
                for (var i = 0; i < res.rows.length; i++) {
                    var item = res.rows.item(i);
                    result.push(item);
                }
            }
            $ionicLoading.hide();
            q.resolve(result);
        }, function (err) {
            $ionicLoading.hide();
            q.reject(err);
        });
        return q.promise;
    }

    //Get a category name from its identificator
    self._getCategoryName = function (categoryId) {
        var q = $q.defer();
        var query = "SELECT name FROM Categories WHERE categoryId = ?;";
        var result = null;

        ToolService.showLoading("LOADING_DATA");
        mallistaDB.executeSql(query, [categoryId]).then(function (res) {
            if (res.rows.length > 0) {
                result = res.rows.item(0).name;
            }
            $ionicLoading.hide();
            q.resolve(result);
        }, function (err) {
            $ionicLoading.hide();
            q.reject(err);
        });
        return q.promise;
    }

    //Get all activated establishments
    self._getEstablishmentList = function (noneItem, fullList) {
        var q = $q.defer();
        var query = "SELECT establishmentId AS id, name FROM Establishments WHERE {0} ORDER BY name COLLATE NOCASE ASC;";
        var where = "activated = 1";
        var result = [];

        if (noneItem != undefined && noneItem == true) {
            var none = { id: null, name: TranslatorService.getToken("NONE") };
            result.push(none);
        }

        if (fullList != undefined && fullList == true)
            where = "1 = 1";

        query = TranslatorService.format(query, where);

        ToolService.showLoading("LOADING_DATA");
        mallistaDB.executeSql(query).then(function (res) {
            if (res.rows.length > 0) {
                for (var i = 0; i < res.rows.length; i++) {
                    var item = res.rows.item(i);
                    result.push(item);
                }
            }
            $ionicLoading.hide();
            q.resolve(result);
        }, function (err) {
            $ionicLoading.hide();
            q.reject(err);
        });
        return q.promise;
    }

    //Get a brand name from its identificator
    self._getEstablishmentName = function (establishmentId) {
        var q = $q.defer();
        var query = "SELECT name FROM Establishments WHERE establishmentId = ?;";
        var result = null;

        ToolService.showLoading("LOADING_DATA");
        mallistaDB.executeSql(query, [establishmentId]).then(function (res) {
            if (res.rows.length > 0) {
                result = res.rows.item(0).name;
            }
            $ionicLoading.hide();
            q.resolve(result);
        }, function (err) {
            $ionicLoading.hide();
            q.reject(err);
        });
        return q.promise;
    }

    //Get all activated products
    self._getProductList = function (fullList) {
        var q = $q.defer();
        var query = "SELECT productId AS id, brandId, categoryId, code," +
                    " strftime('%d/%m/%Y %H:%M:%S', creationDate) AS creationDate," +
                    " establishmentId, image," +
                    " strftime('%d/%m/%Y %H:%M:%S', modificationDate) AS modificationDate," +
                    " name, remarks" +
                    " FROM Products" +
                    " WHERE {0}" +
                    " ORDER BY name COLLATE NOCASE ASC;";
        var result = [];
        var where = "activated = 1";

        if (fullList != undefined && fullList == true)
            where = "1 = 1";

        query = TranslatorService.format(query, where);

        ToolService.showLoading("LOADING_DATA");
        mallistaDB.executeSql(query).then(function (res) {
            if (res.rows.length > 0) {
                for (var i = 0; i < res.rows.length; i++) {
                    var item = res.rows.item(i);
                    result.push(item);
                }
            }
            $ionicLoading.hide();
            q.resolve(result);
        }, function (err) {
            $ionicLoading.hide();
            q.reject(err);
        });
        return q.promise;
    }

    //Get a product name from its identificator
    self._getProductName = function (productId) {
        var q = $q.defer();
        var query = "SELECT name FROM Products WHERE productId = ?;";
        var result = null;

        ToolService.showLoading("LOADING_DATA");
        mallistaDB.executeSql(query, [productId]).then(function (res) {
            if (res.rows.length > 0) {
                result = res.rows.item(0).name;
            }
            $ionicLoading.hide();
            q.resolve(result);
        }, function (err) {
            $ionicLoading.hide();
            q.reject(err);
        });
        return q.promise;
    }

    //Indicate whether a product exists on database
    self._isRegistered = function (name) {
        var q = $q.defer();
        var query = "SELECT COUNT(*) AS total FROM Products WHERE name LIKE ?;";
        var result = false;

        ToolService.showLoading("LOADING_DATA");
        mallistaDB.executeSql(query, [name]).then(function (res) {
            $ionicLoading.hide();
            if (res.rows.length > 0) {
                result = res.rows.item(0).total > 0;
            }
            q.resolve(result);
        }, function (err) {
            $ionicLoading.hide();
            q.reject(err);
        });
        return q.promise;
    }

    //Update a specific product
    self._updateProduct = function (id, brandId, categoryId, code, establishmentId, image, name, remarks) {
        var q = $q.defer();
        var query = "UPDATE Products SET brandId = ?, categoryId = ?, code = ?, establishmentId = ?, image = ?,  name = ?, remarks = ?, modificationDate = datetime('now', 'localtime') WHERE productId = ?;";

        brandId = ToolService.getValueOrNull(brandId);
        categoryId = ToolService.getValueOrNull(categoryId);
        establishmentId = ToolService.getValueOrNull(establishmentId);

        ToolService.showLoading("SAVING_DATA");
        mallistaDB.executeSql(query, [brandId, categoryId, code, establishmentId, image, name, remarks, id], true).then(function (res) {
            $ionicLoading.hide();
            q.resolve();
        }, function (err) {
            $ionicLoading.hide();
            if (err.code == 19)
                err.message = TranslatorService.getToken("ERROR_REGISTERED", name);
            ToolService.showMessage(err.message);
            q.reject(err);
        });
        return q.promise;
    }
    //Service's public method
    return {
        addProduct: function (brandId, categoryId, code, establishmentId, image, name, remarks) {
            return self._addProduct(brandId, categoryId, code, establishmentId, image, name, remarks);
        },
        deleteProduct: function (id){
            return self._deleteProduct(id);
        },
        getBrandList: function (noneItem) {
            return self._getBrandList(noneItem);
        },
        getBrandName: function (brandId) {
            return self._getBrandName(brandId);
        },
        getCategoryList: function (noneItem) {
            return self._getCategoryList(noneItem);
        },
        getCategoryName: function (categoryId) {
            return self._getCategoryName(categoryId);
        },
        getEstablishmentList: function (noneItem) {
            return self._getEstablishmentList(noneItem);
        },
        getEstablishmentName: function (establishmentId) {
            return self._getEstablishmentName(establishmentId);
        },
        getFullBrandList: function (noneItem) {
            return self._getBrandList(noneItem, true);
        },
        getFullCategoryList: function (noneItem) {
            return self._getCategoryList(noneItem, true);
        },
        getFullEstablishmentList: function (noneItem) {
            return self._getEstablishmentList(noneItem, true);
        },
        getFullProductList: function () {
            return self._getProductList(true);
        },
        getProductList: function () {
            return self._getProductList();
        },
        getProductName: function (productId) {
            return self._getProductName(productId);
        },
        isRegistered: function (name) {
            return self._isRegistered(name);
        },
        updateProduct: function (id, brandId, categoryId, code, establishmentId, image, name, remarks) {
            return self._updateProduct(id, brandId, categoryId, code, establishmentId, image, name, remarks);
        },
    };
}])
//Filter used with product list
.filter('productFilter', [function () {
    return function (items, expression) {
        var foundItems = [];

        if (expression != undefined) {
            expression = expression.toLowerCase();

            angular.forEach(items, function (item) {
                if (item.name != undefined) {
                    if (item.name.toLowerCase().indexOf(expression) >= 0)
                        foundItems.push(item);
                }
            });
        }
        return foundItems;
    };
}])
;


